#! /usr/bin/python -m

VERSION='1.5.20201106a'
#-- imports -- 
import os
import sys
import socket
import ctypes
import re
import json
import pickle
import logging
import collections
import array
import platform
import pprint
import glob
import urllib2
import requests 
from shutil import copy as shutil_copy
import threading
import logging
from datetime import datetime,timedelta
from time import ctime
from flask import Flask, Response,session,  flash,  render_template, url_for, abort, redirect, request,  g,send_file, jsonify
import flask_login
import ConfigParser
from pyorbital.orbital import Orbital
import numpy as np 
from math import sqrt,atan2


import sdr


VALID_INI_ID = ['xxx']
VALID_LOG_ID = ['xxx']
#--- constants
TLE_UPDATE_PERIOD_H = 8 #hours
SCHEDULE_EXTENT = 4 #days
RT_SCHEDULER_TICK = 5 #seconds
CALC_MIN_ELEV = 30  #should not be very high to ensure passes longer 1 min, this is only seed for pass detection, noDEt for rise-set times
PASS_RUN_BEFORE_ENTRANCE = 5 #seconds, start recevier ( run_pass()) before scheduled time
SCHED_OVERLAP_MARGIN = 60  #margin between passes to count them as overlapping (seconds)
	
#-- config --
APP_ROOT = os.path.abspath(os.path.dirname(__file__))
APP_ROOT_TOP = os.path.dirname(APP_ROOT)
conf_file = os.path.join(APP_ROOT, 'config', 'lora.conf')
LOG_FILE = os.path.join(APP_ROOT,'var','log','lora.log')
LORETT_EUS_URL='http://eus.lorett.org/eus/log_post'

# base configuration template
CONF = { 
		
		'station_id' : 'not_configured' ,
		'station_lat' : 0.0, 
		'station_lon' :  0.0, 
		
		'station_alt' : 0.0, #km
		'min_elevation' : 60.0,
		'pass_entry_time' : 60,
		'allow_online_support' : 1,
		'keep_free_space' : 80 * 1000000, #kB
		
		'satlist' : ['NOAA 18','NOAA 19','METOP-B','METOP-C','FENGYUN 3C','METEOR-M 2', 'METEOR-M2 2'],
		'track_export' : os.path.join(APP_ROOT_TOP,'tracks','Track.txt'),
		'track_archive' : os.path.join(APP_ROOT_TOP,'tracks_archive'),
		'rec_archive': os.path.join(APP_ROOT_TOP,'tracks_archive'),
		'tm_data_dir': os.path.join(APP_ROOT_TOP,'data'),
		'machines' : [ ], 		
		'tle_path' : APP_ROOT + '/var/tle',
		'debug' : True		,
		'receiver' : {
				'type' : 'sdr',
				'soapy_source' : 'airspy'
			},
		'USERS': {'admin' : { 'pw' : 'alpha', 'is_admin' : True} },
		"data_path": "C:\\lorett\\data"
		} #default seed config
#-- update from config file
try:
	add_conf = json.load(file(conf_file))
	CONF.update(add_conf)
except Exception as e:
	sys.stderr.write("WARNING: failed to load config from [%s]:\n%s\n" % (conf_file, str(e)))

print CONF
#-- some globals 
g_start_time = datetime.utcnow()
SAT_PARAMS = {
		'AQUA' : { 'id':'PM1', 'rec_config' : 'AQUA'}, 
		'TERRA' : { 'id':'AM1', 'rec_config' : 'TERRA'}, 
		'SUOMI NPP' : { 'id':'NPP', 'rec_config' : 'NPP'}, 
		#'SPOT 6' : { 'id':'SP6', 'rec_config' : 'SPOT67'}, 
		#'SPOT 7' : { 'id':'SP7', 'rec_config' : 'SPOT67'}, 
		'RESURS P3' : { 'id':'RP3', 'rec_config' : 'RESUPSP3'}, 
		'RESURS P2' : { 'id':'RP2', 'rec_config' : 'RESURSP2'}, 
		'FENGYUN 3C' : { 'id':'F3C', 'rec_config' : 'FENGYUN 3C'}, 
		'FENGYUN 3B' : { 'id':'F3B', 'rec_config' : 'FENGYUN 3B'},
		'FENGYUN 3D' : { 'id':'F3D', 'rec_config' : 'FENGYUN 3D'}
	
	}



TLE_SOURCES = {
	'default': [
		#'http://www.celestrak.com/NORAD/elements/science.txt',
		#'http://www.celestrak.com/NORAD/elements/resource.txt',
		'http://www.celestrak.com/NORAD/elements/weather.txt'
		#'http://eostation.scanex.ru/schedule/tle/new.tle'
		]
}

CLEANSPACE_GLOB = '*.iq'

#--
RADIO = None
SCHED = []
SCANNING_TRACK = False
#tracking alg = ['az','cross','off']
TRACK_SCAN = {'alg':'off','bias_step': 0.999, 't_start':0.0,'t_step':0.0, 'el_range' : 3.2,'el_step':0.8  }
BIAS_DTIME = 0
BIAS_DAZ = 0
CAL_MATRIX = None 


def get_disk_space_kb(dirname):
	"""Return folder/drive free space (in Kibibytes)."""
	if platform.system() == 'Windows':
		free_bytes = ctypes.c_ulonglong(0)
		total_bytes = ctypes.c_ulonglong(0)
		ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(dirname), None, ctypes.pointer(total_bytes), ctypes.pointer(free_bytes))
		return free_bytes.value /1024
	else:
		st = os.statvfs(dirname)
		return st.f_bavail * st.f_frsize /1024

def clean_space(dirname,need_space):
	""" removes oldest files untill required free space will left on the dirve.
		- returns left free space (kB)
	"""
	space = get_disk_space_kb(dirname)
	print "Space check on {} = {} <> {}".format(dirname, space , need_space)
	if space > need_space: return space #no need to clean
	try:
		files = list(filter(os.path.isfile, glob.glob(os.path.join(dirname,CLEANSPACE_GLOB))))
		files.sort(key=lambda x: os.path.getmtime(x), reverse=False)
		for f in files:
			fs = os.path.getsize(f)/1024
			print "removing old file: {} {}  {}kB".format(f,os.path.getmtime(f),fs)
			os.remove(f)
			space += fs
			if space > need_space: break
	except Exception as e:
		print "clean_space exception: {}".format(e)
	return space

def TZ(dt_utc):
	""" Convert UTC datetime to local datetime using either system of configured TZ """
	tz = datetime.now() - datetime.utcnow()
	return dt_utc + tz
	
def get_tle(satname=None,t0 = None):
	if CONF['tle_path'] is not None:
		tle = CONF['tle_path']+"/new.tle"
	else:
		tle = None
	return tle


def post_file_to_lorett_support(fqfn,ftype='test'):
	if not ( 1 == CONF['allow_online_support']) : return #online support is not allows by config
	#
	try:
		form = {'station': CONF['station_id'], 'ftype': ftype}
		files = {"file": open(fqfn, "rb")}
		r = requests.post(LORETT_EUS_URL, files=files, data=form)
		if 200 == r.status_code: logmsg("uploaded log {}".format(os.path.basename(fqfn)))
		else :logmsg("log upload failed [{}], responce: {}".format(r.status_code,r.text))
	except Exception as e:
		logmsg("EXCEPTION in log upload: {}".format(e))
	return True
	
def format_almur_schedule(S):
	""" format a given schedule to text of ALMUR schedule.ini """
	ipass=0
	result = ''
	for r in S:
		result +=  "\n[%d]\n" % (ipass)
		result += "description=%s\n" %( r['satellite'] ) 
		t = TZ(r['start'] - timedelta(seconds=10))
		result += "start=%s\n" %( t.strftime('%Y.%m.%d %H:%M:%S') ) 
		t = TZ(r['end'] + timedelta(seconds=10))
		result += "stop=%s\n" %( t.strftime('%Y.%m.%d %H:%M:%S') ) 
		result  +="config=%s\n" %(CONF['almur_ini_path'] + "/"+  r['satellite'] + '.ini'  )
		#
		ipass = ipass+1
	return result
	
#-============================= angle calibration block

PI = 3.1416
DEG2RAD = 3.1416/180.0
RAD2DEG = 1.0/DEG2RAD

def utime(dt):
	''' datetime to unix timestamp '''
	return (dt - datetime(1970,1,1)).total_seconds()
	
def azel2vec(azel): #converts [az,el] to [x,y,z] unit vectors, also may convert from degrees
	x = np.sin(PI/2.0 - azel[1]) * np.cos(azel[0])
	y = np.sin(PI/2.0 - azel[1]) * np.sin(azel[0])
	z = np.cos(PI/2.0 - azel[1])
	return np.array([x,y,z]) #

def vec2azel(xyz):
	XsqPlusYsq = xyz[0]**2 + xyz[1]**2
	r = sqrt(XsqPlusYsq + xyz[2]**2)               # r
	elev = atan2(xyz[2],sqrt(XsqPlusYsq))     # theta
	az = atan2(xyz[1],xyz[0])                           # phi
	return az,elev

def to_vector(X):
	A = np.zeros([X.shape[0],3])
	for i in range(X.shape[0]):
			A[i,:] = azel2vec(X[i,:])
	return A
	

def calibrate(az,el):
			
	in_vec = azel2vec( [az*DEG2RAD,el*DEG2RAD] )
	out_vec = np.dot(CAL_MATRIX , in_vec.T)
	out_az,out_el = vec2azel(out_vec)
	return out_az*RAD2DEG,out_el*RAD2DEG
#-============================= angle calibration block	
def get_track_text2(satname,t0 = None,t1=None):
	''' Calculate track to pass to veg driver '''
	track_txt = ''
	if t0 is None:
		t0 = datetime.utcnow()
	tle = get_tle(satname,t0)
	orb = Orbital(satname, tle_file=tle)
	
	print "-get_track_text(): {} - {} - {}".format(satname,t0,t1)
	passes = orb.get_next_passes(t0-timedelta(minutes=20),72,CONF['station_lon'],CONF['station_lat'],CONF['station_alt'],tol=0.01, horizon=CALC_MIN_ELEV) 
	
	if len(passes)> 0:
		next_pass =  passes[0]
		pass_start = next_pass[0].replace(microsecond=0)
		pass_end = next_pass[1].replace(microsecond=0)
		orbnum = orb.get_orbit_number(next_pass[2])
		#
		track_txt += "Satellite: {}\n".format(satname)
		
		#
		dt=0
		t = pass_start + timedelta(seconds=dt)
		nline = 0
		while t < pass_end:
			az, el = orb.get_observer_look(t, CONF['station_lon'],CONF['station_lat'], 0)
			azc,elc = az, el
			if CAL_MATRIX is not None:
				azc,elc = calibrate(az,el)
				if azc < 0: azc = azc +360.0
			
			if el > CONF['min_elevation']:
				nline += 1
				if nline == 1:
					track_txt += "Start date & time: {:%Y-%m-%d}   {:%H:%M:%S} UTC\n".format(t,t)
					track_txt += "Orbit: {}\n".format(orbnum)
					track_txt += "\n"
					track_txt += "Time (UTC)   Azimuth (deg:min)   Elevation (deg:min)\n"
					track_txt += "\n"
				track_txt += "{}   {:.3f}   {:.3f}   {:.3f}   {:.3f}\n".format(t.strftime("%H:%M:%S"),az,el,azc,elc)
			#
			dt+=1
			t = pass_start + timedelta(seconds=dt)
	else:
		print "WARNING: no first pass ffound for {} at {}".format(satname,t0)
		return None
	return track_txt

	
def get_track_text(satname,t0 = None,t1=None,scan_track=False):
	''' Calculate track to pass to veg driver '''
	track_txt = ''
	bias_txt = ''
	track_start = datetime(2000,1,1,0,0,0)
	if t0 is None:
		t0 = datetime.utcnow()
		
	tle = get_tle(satname,t0)
	orb = Orbital(satname, tle_file=tle)
	
	print "-get_track_text(): {} - {} - {}".format(satname,t0,t1)
	passes = orb.get_next_passes(t0-timedelta(minutes=20),72,CONF['station_lon'],CONF['station_lat'],CONF['station_alt'],tol=0.01, horizon=CALC_MIN_ELEV) 
	track_arr = []
	
	if len(passes)> 0:
		next_pass =  passes[0]
		pass_start = next_pass[0].replace(microsecond=0)
		pass_end = next_pass[1].replace(microsecond=0)
		orbnum = orb.get_orbit_number(next_pass[2])
		#
		track_txt += "Satellite: {}\n".format(satname)
		#
		TRACK_STEP = 1.0
		dt=0.0
		t = pass_start + timedelta(seconds=dt)
		nline = 0
		#-- shifts --
		bias_t = 0.0
		bias_el = 0.0
		bias_az = 0.0
		bias_cross = 0.0
		bias_cross_v = 1.0
		bias_prev_dt = 0.0
		if scan_track:
			bias_t =  TRACK_SCAN['t_start']
			bias_cross = -1.0* TRACK_SCAN['el_range']
		f_track_started = False
		while t < pass_end:
			#adjust bias 
			if f_track_started and dt - bias_prev_dt > TRACK_SCAN['bias_step']:
				bias_cross += bias_cross_v * TRACK_SCAN['el_step']
				if bias_cross > TRACK_SCAN['el_range']: 
					bias_cross_v = -1.0
					bias_t += TRACK_SCAN['t_step'] #step along track at each cross-track turn
				elif bias_cross < -1.0 * TRACK_SCAN['el_range']: 
					bias_cross_v = 1.0
					bias_t += TRACK_SCAN['t_step']
				bias_prev_dt = dt
			#-- orbit calculation time
			if scan_track: t_calc = t + timedelta(seconds=bias_t)
			else: t_calc = t
			#
			az_orbit, el_orbit = orb.get_observer_look(t_calc + timedelta(seconds=BIAS_DTIME), CONF['station_lon'],CONF['station_lat'], 0)
			az, el = az_orbit, el_orbit
			#-- apply calibration rotation
			if CAL_MATRIX is not None:
				az,el = calibrate(az,el)
				if az < 0: az = az +360.0
			track_arr.append([az,el])
			#
			#
			if scan_track: 
				if TRACK_SCAN['alg'] == 'az': #simple azimuth using cross bias
					bias_az = bias_cross
					bias_el = 0.0
				elif TRACK_SCAN['alg'] == 'cross': #
					if True and len(track_arr) > 2: # approx. cross-track direction
						r = (90.0-el)
						if r < 2: r = 2
						r *= 1.0/10.0
						vel_az = az - track_arr[-2][0]
						vel_el = el - track_arr[-2][1]
						vel_norm = sqrt((vel_az/r)*(vel_az/r) + vel_el*vel_el)
						cross_az =  r* vel_el / vel_norm #unit vector in cross direction
						cross_el =  -1.0 * (vel_az/r) / vel_norm
						bias_el = bias_cross * cross_el
						bias_az = bias_cross * cross_az
					else: #simple bias az/el
						bias_el = bias_cross
						#bias_az = bias_cross
				else: #no scan
					bias_az = 0.0
					bias_el = 0.0
				#--
				az += bias_az
				el += bias_el			
			#
			#constant bias applied always	
			az += BIAS_DAZ
			bias_az += BIAS_DAZ
			if az < 0.0 : az += 360.0
			if az >360.0 : az -= 360.0		
			if el >90.0: el = 90.0 #may happen after simple addition
			#
			if el_orbit > CONF['min_elevation']:
				nline += 1
				if nline == 1:
					f_track_started = True
					track_start = t
					track_txt += "Start date & time: {:%Y-%m-%d}   {:%H:%M:%S} UTC\n".format(t,t)
					track_txt += "Orbit: {}\n".format(orbnum)
					track_txt += "\n"
					track_txt += "Time (UTC)   Azimuth (deg:min)   Elevation (deg:min)\n"
					track_txt += "\n"
					
					bias_txt = "Bias dt={}, dza={}\nScan track: {}\nTLE:\n{}\n\n".format(BIAS_DTIME,BIAS_DAZ,scan_track,get_tle_set(satname,None))
					bias_txt += track_txt
					bias_txt +="Time\tTargetAz\tTargetEl\tOrbitAz\tOrbitEl\tbias_t\tbias_cross\tbias_az\tbias_el\n"
				track_txt += "{}   {:03}:{:02}   {:03}:{:02}\n".format(t.strftime("%H:%M:%S"),int(az),int((az - int(az))*60),int(el),int((el - int(el))*60))
				bias_txt += "{:%H:%M:%S.%f}\t{:.2f}\t{:.2f}\t{:.2f}\t{:.2f}\t{:.2f}\t{:.2f}\t{:.2f}\t{:.2f}\n".format(t,az,el,az_orbit, el_orbit,bias_t,bias_cross,bias_az,bias_el)
			else:
				bias_prev_dt = dt #do not adjust bias if no tracking
			#
			dt+=TRACK_STEP
			t = pass_start + timedelta(seconds=dt)
	else:
		print "WARNING: no first pass found for {} at {}".format(satname,t0)
		return None
	return track_txt,track_start,bias_txt

def get_sat_schedule(satname,t0 = None,t1=None):
	''' Calculate track to pass to veg driver '''
	track_txt = ''
	if t0 is None:
		t0 = datetime.utcnow()
	if t1 is None:
		t1 = t0+timedelta(days=SCHEDULE_EXTENT)
		#t1 = t1.replace(second=0,microsecond=0)
	duration = 24*SCHEDULE_EXTENT
	tle = get_tle(satname,t0)
	orb = Orbital(satname, tle_file=tle)
	passes = orb.get_next_passes(t0,duration ,CONF['station_lon'],CONF['station_lat'],CONF['station_alt'],tol=0.01, horizon=CALC_MIN_ELEV) 
	#
	S = []
	for next_pass in passes:
		pass_start = next_pass[0]
		pass_end = next_pass[1]
		pass_culm = next_pass[2]
		caz, cel = orb.get_observer_look(pass_culm, CONF['station_lon'],CONF['station_lat'], 0)
		if cel < CONF['min_elevation']:
			continue ## drop pases below elevation of interst
		#--get true AOS-LOS at mask
		dT = 0
		pass_los = None
		pass_aos = None
		t = pass_start
		while t < pass_end:
			t = pass_start+timedelta(seconds=dT)
			az, el = orb.get_observer_look(t, CONF['station_lon'],CONF['station_lat'], 0)
			if el > CONF['min_elevation']:
				pass_los = t
				if pass_aos is None: pass_aos = t
			dT += 3
		if pass_los is None or pass_aos is None:
			print "Warning: not detected AOS[{}]/LOS[{}]: {}/{}. culm={}".format(pass_aos,pass_los,pass_start,pass_end,cel)
			continue #skip this pass
		orbnum = orb.get_orbit_number(pass_culm)
		#
		passid = "{:%Y%m%d_%H%M%S}_{}".format(pass_aos,satname)
		S.append({'passid': passid, 'start': pass_aos, 'end': pass_los, 'culm': pass_culm, 'culm_el': cel, 'orbit':orbnum , 'satellite':satname, 
				'state': 'onlist' })
	#
	return S
	
def join_schedules(S1,S2):
	S1 += S2
	#and resort it
	S1.sort(key=lambda x: x['start'], reverse=False)
	return

def calc_schedule(t0=None, t1=None):
	S = []
	for sat in CONF['satlist']: #collect schedules sat by sat
		ssat = get_sat_schedule(sat,t0=t0,t1=t1)
		S += ssat
	S.sort(key=lambda x: x['start'], reverse=False)
	#
	return S

def format_schedule(satnames = [],t0=None, t1=None):
	""" format schedule in html table """
	S = []
	for sat in satnames: #collect schedules sat by sat
		ssat = get_sat_schedule(sat,t0=t0,t1=t1)
		S += ssat
		S.sort(key=lambda x: x['start'], reverse=False)
	#
	stable = "<table class='schedule'>\n<tr><th>Satellite</th><th>Pass rise (UTC)</th><th>Set</th><th>Orbit</th><th>Culmination</th></tr>\n"
	for i in range(len(S)): #for pass in schedule
		p = S[i]
		overlap = ''  # add radio buttons to double overlaps
		selected = ''
		conflict_class = ''
		if i < len(S)-1 and S[i+1]['start'] < p['end'] + timedelta(seconds = SCHED_OVERLAP_MARGIN) : #first in conflicting pair
			selected = '' if BANNED_PASSES.is_banned(p['passid']) else 'checked'
			overlap = '<input class="rb" type="radio" id="{}" name="{}" value="{}"  onclick="on_conflict(this);" {} >'.format(p['passid'],p['passid'],p['passid'],selected)
			conflict_class = 'conflict'
		if i > 0 and p['start'] < S[i-1]['end'] + timedelta(seconds = SCHED_OVERLAP_MARGIN) : #second 
			selected = 'checked' if BANNED_PASSES.is_banned(S[i-1]['passid']) else ''
			overlap = '<input class="rb" type="radio" id="{}" name="{}" value="{}"  onclick="on_conflict(this);" {} >'.format(p['passid'],S[i-1]['passid'],p['passid'],selected)
			conflict_class = 'conflict'
		#
		if p['end'] < datetime.utcnow(): #PAST
			past_banned = 'past_banned' if BANNED_PASSES.is_banned(p['passid']) else ''
			rec_log_url = "/viewfile/{:%Y%m%d_%H%M%S}_{}_rec.log".format(p['start'],p['satellite'])
			track_log_url = "/viewfile/{:%Y%m%d_%H%M%S}_{}_bias_pass.txt".format(p['start'],p['satellite'])
			joined_url = "/joinlog/{:%Y%m%d_%H%M%S}_{}.log".format(p['start'],p['satellite'])
			tview_url = "/trackview/{:%Y%m%d_%H%M%S}_{}".format(p['start'],p['satellite'])
			stable += "<tr><td class='ra {past_banned} {conflict_class}'>{satname}</td><td style='background-color: #CCCCCC;'>{start:%Y-%m-%d %H:%M:%S}</td><td>{end:%H:%M:%S}</td><td>{orbit}</td><td>{culm:.1f}</td><td><a href='export_track/{satname}/{start:%Y-%m-%d %H:%M:%S}'>old track</a> | <a href='{track_log}' target=_blank> track.log </a> | <a href='{rec_log}' target=_blank> rec.log </a> | <a href='{tview_url}' target=_blank> track view </a></td></tr>\n".format(satname=p['satellite'],start=p['start'],end=p['end'],orbit=p['orbit'],culm=p['culm_el'],track_log=track_log_url,rec_log=rec_log_url,tview_url=tview_url,conflict_class=conflict_class,past_banned=past_banned)
		elif p['start'] < datetime.utcnow(): #CURRENT
			stable += "<tr><td class='ra {conflict_class}'>{satname} {overlap}</td><td style='background-color: green'>{start:%Y-%m-%d %H:%M:%S}</td><td>{end:%H:%M:%S}</td><td>{orbit}</td><td>{culm:.1f}</td><td><a href='export_track/{satname}/{start:%Y-%m-%d %H:%M:%S}'>straight track</a> | <a href='export_track/{satname}/{start:%Y-%m-%d %H:%M:%S}?scan=1'>scan track</a> | <a href='{tview_url}' target=_blank> track view </a></td></tr>\n".format(satname=p['satellite'],start=p['start'],end=p['end'],orbit=p['orbit'],culm=p['culm_el'],tview_url=tview_url,conflict_class=conflict_class,overlap=overlap)
		else: #FUTURE
			stable += "<tr><td class='ra {conflict_class}'>{satname} {overlap}</td><td>{start:%Y-%m-%d %H:%M:%S}</td><td>{end:%H:%M:%S}</td><td>{orbit}</td><td>{culm:.1f}</td><td><a href='export_track/{satname}/{start:%Y-%m-%d %H:%M:%S}'>save track</a>  | <a href='export_track/{satname}/{start:%Y-%m-%d %H:%M:%S}?scan=1'>scan track</a> | <a href='{tview_url}' target=_blank> track view </a></td></tr>\n".format(satname=p['satellite'],start=p['start'],end=p['end'],orbit=p['orbit'],culm=p['culm_el'],tview_url=tview_url,conflict_class=conflict_class,overlap=overlap)
	stable += "</table>\n"
	
	return { 'is_empty' : (len(S)<1), 'table': stable}

def run_post_pass(pass_to_run):
	global SCHED
	pass_id = pass_to_run['passid'] 
	logmsg("End of pass {}, los at {}".format(pass_to_run['passid'],pass_to_run['end']))
	pass_to_run['state']='finished'
	#-- stop receiver
	##no need, use global initialized at pass start- R = almurnet.ALMUR_DEM('localhost',CONF['receiver']['man_port'])
	global RADIO
	if RADIO is not None: 
		res = RADIO.stop()
		if hasattr(RADIO,'rssi_log'): #save RSSI log
			try:
				rssi_file_name =os.path.join(CONF['rec_archive'],pass_id+'_rec.log') 
				tle = get_tle(pass_to_run['satellite'],pass_to_run['end'])
				orb = Orbital(pass_to_run['satellite'], tle_file=tle)
				with open(rssi_file_name, "wt") as rssi_f:
					rssi_f.write("#Pass ID: {}\n".format(pass_id))
					rssi_f.write("#Satellite: {}\n".format(pass_to_run['satellite']))
					rssi_f.write("#Configuration: {}\n".format(pass_to_run['satellite']))
					rssi_f.write("#Lora version: {}\n".format(VERSION))
					rssi_f.write("#TLE: {}\n".format(get_tle_set(pass_to_run['satellite'])))
					rssi_f.write("#Station: {} at {},{}\n".format(CONF['station_id'],CONF['station_lon'],CONF['station_lat']))
					rssi_f.write("#Start time: {}\n\n".format(pass_to_run['start']))
					rssi_f.write("#Time\tAz\tEl\tLevel\tSNR\n")
					for r in RADIO.rssi_log:
						t = r[0]
						az, el = orb.get_observer_look(t, CONF['station_lon'],CONF['station_lat'], 0)
						rssi_f.write("{}\t{:.1f}\t{:.1f}\t{:.1f}\t{:.1f}\n".format(t,az,el,r[1],r[2]))
				post_file_to_lorett_support(rssi_file_name,ftype='sdr_rssi_log')
			except Exception as ee:
				print "ERROR: RSSI log save exception for pass {}: {}".format(pass_id,ee)
		RADIO = None 
		logmsg("Receiver stoppped, {}".format(res))
	#--- update the schedule
	#SCHED = calc_schedule(t0=datetime.utcnow() - timedelta(seconds=10*60),t1=datetime.utcnow() + timedelta(hours=2*24))
	schedule_next_pass(None)
	return True
	

def run_pass(pass_to_run):
	pass_id = pass_to_run['passid'] 
	logmsg("Running pass {}, aos at {}, los at {}".format(pass_to_run['passid'],pass_to_run['start'],pass_to_run['end']))
	pass_to_run['state'] = 'running'
	#--schedule next event -> post_pass
	RTS.add_event(pass_to_run['end']+timedelta(seconds=10),ev_description="{}. LOS of {}".format(pass_to_run['satellite'],pass_to_run['orbit']),
		ev_handler=run_post_pass,handler_arg=pass_to_run,attr=pass_to_run)
	#
	#--- prepare and run receiver
	global RADIO
	radio_configured = False
	tm_log = ''
	tm_data = ''
	tm_spectr = ''
	try:
		if CONF['receiver']['type'] == 'sdr':
			#RADIO = sdr.OSMO_SDR(CONF['receiver']['soapy_source'])  == initialized in pre-pass
			radio_configured = RADIO.load_config(pass_to_run['satellite'])
			tm_log = ''
			tm_data = os.path.join(CONF['tm_data_dir'],pass_id+'.iq')
			tm_spectr = os.path.join(CONF['rec_archive'],pass_id+'_spectr.log')
			logmsg("RADIO reload {}, {}, config={}".format(CONF['receiver']['type'],CONF['receiver']['soapy_source'],pass_to_run['satellite']))
		elif CONF['receiver']['type'] == 'dem':
			RADIO = almurnet.ALMUR_DEM('localhost',CONF['receiver']['man_port'])
			rec_conf = SAT_PARAMS[pass_to_run['satellite']]['rec_config']
			load_result = DEM_RECEIVER.load_config( config_id=rec_conf)
			logmsg("Loaded receiver config: {}, {}".format(rec_conf,load_result))
		else:
			logmsg("ERROR: invalid radio type [{}]".format(CONF['receiver']['type']))
		#
		if radio_configured:
			start_result = RADIO.start(data_file=tm_data,log_file=tm_log,spectr_file=tm_spectr)
			logmsg("Receiver started: {}".format(start_result))
		else:
			logmsg("ERROR: radio configuration failed")
		#--run the receiver logger
		#log_duration = pass_to_run['end'] - datetime.utcnow()
		#log_duration += timedelta(seconds=30)	
		#logmsg("Running TM log till {} to {}".format(pass_to_run['end'],tm_log))
		#rec_thread = DEM_RECEIVER.log_telemetry_threaded(passid=pass_id,
		#			satellite=pass_to_run['satellite'], config=rec_conf, stop_time = pass_to_run['end'],
		#			log_file= tm_log)
		
	except Exception as e:
		print "EXCEPTION in radio preparation for {}: {}".format(pass_id,e)
	return True
	
def run_pass_prepare(pass_to_run):
	pass_id = pass_to_run['passid'] 
	logmsg("Running pass prepare {} at {}".format(pass_id,pass_to_run['start']))
	pass_to_run['state'] = 'preparing'
	#-- check times 
	if pass_to_run['end'] < datetime.utcnow():
		logmsg("ERROR: preparing pass in the past {}".format(pass_to_run['passid']))
		return
	#---- prepare antenna
	track_txt,track_start,track_bias = get_track_text(pass_to_run['satellite'],pass_to_run['start']-timedelta(minutes=15),pass_to_run['end']+timedelta(minutes=15),scan_track=SCANNING_TRACK)
	if track_txt is not None:
		if 'track_export' in CONF:
			try:
				with open(CONF['track_export'], "w") as trackfile:
					trackfile.write(track_txt)
					logmsg("Saved track for pass " + pass_id)
				with open(os.path.join(CONF['track_archive'],pass_id+'.txt'), "w") as trackfile:
					trackfile.write(track_txt)
				with open(os.path.join(CONF['track_archive'],pass_id+'_bias_pass.txt'), "w") as trackfile:
					trackfile.write(track_bias)
			except Exception as e:
				logmsg("ERROR saving track for pass {}, error:{}".format(pass_id,str(e)))
		if CONF['receiver']['type'] == 'sdr': #do preparation at pre-pass step (noise calibration)
			global RADIO
			try:
				RADIO = sdr.OSMO_SDR(CONF['receiver']['soapy_source'])
				radio_configured = RADIO.load_config(pass_to_run['satellite'])
				RADIO.calibrate(pass_to_run['satellite'])
				logmsg("SDR calibration level:{:.1f}".format(RADIO.noise_level))
				logmsg("RADIO setup {}, {}, config={}".format(CONF['receiver']['type'],CONF['receiver']['soapy_source'],pass_to_run['satellite']))
			except Exception as e:
				logmsg("ERROR preconfiguring radio for {}, error:{}".format(pass_id,str(e)))
	try:
		update_tle()
	except Exception as e:
		logmsg("Eception in call to update TLE before pass:{}".format(e))
	try:
		if 'keep_free_space' in CONF:
			free_space = clean_space(CONF['tm_data_dir'],CONF['keep_free_space'])
			logmsg("disk free space: {} GB".format(free_space)/1000000 )
	except Exception as e:
		logmsg("Eception in prepass clean:{}".format(e))
	#-- schedule next events
	run_at = pass_to_run['start']-timedelta(seconds=PASS_RUN_BEFORE_ENTRANCE)
	if run_at < datetime.utcnow(): #--run it immediately
		run_pass(pass_to_run)
	else: #schedule run at AOS time
		RTS.add_event(run_at,ev_description="{}. AOS of {}".format(pass_to_run['satellite'],pass_to_run['orbit']),
			ev_handler=run_pass, handler_arg = pass_to_run,attr=pass_to_run)
		logmsg("pass {} run scheduled at {:%H:%M:%S}".format(pass_to_run['passid'],run_at))
	#-----------------
	return True
	
def schedule_next_pass(dummy_arg, clean_all_events=False):
	""" Select next pass from the schedule and prepare the station modules to perform it , schedule a next pass after that"""
	global SCHED
	print "-scheduling next pass"
	#
	if clean_all_events: RTS.clean()
	#
	try:
		SCHED = calc_schedule(t0=datetime.utcnow() - timedelta(seconds=15*60),t1=datetime.utcnow() + timedelta(hours=2*24))
	except Exception as ee:
		logmsg("preschedule exception: [{}]".format(ee))
	for next_pass in SCHED:
		if (next_pass['end'] < datetime.utcnow()):  #the pass is over
			next_pass['state'] = 'over'
			print ">> pass over {} , los= {} ".format(next_pass['passid'],next_pass['end'])
			continue
		pass_id = "{:%Y%m%d_%H%M%S}_{}".format(next_pass['start'],next_pass['satellite'])
		print ">> " +pass_id
		if BANNED_PASSES.is_banned(pass_id):
			logmsg("skip banned pass "+pass_id)
			continue
		if next_pass['start']-timedelta(seconds=CONF['pass_entry_time']) > datetime.utcnow() : #then schedule it via RT scheduler -> waiting
			next_pass['state'] = 'waiting'
			RTS.add_event(next_pass['start']-timedelta(seconds=CONF['pass_entry_time']),
				ev_description="{}. pass {}".format(next_pass['satellite'],next_pass['orbit']),
				ev_handler=run_pass_prepare,handler_arg = next_pass,attr=next_pass)
			logmsg("put on schedule pass [{}]".format(pass_id))
			return True#do not check further passes in the schedule
		else:
			logmsg("running immediately [{}]".format(pass_id))
			run_pass_prepare(next_pass)
			return True
	#-- if we are here - no more passes i nthe schedule, let us calculate new
	SCHED = calc_schedule(t0=datetime.utcnow() - timedelta(seconds=15*60),t1=datetime.utcnow() + timedelta(hours=2*24))
	RTS.add_event( datetime.utcnow() + timedelta(seconds=60), #try to repeat scheduling after some time
				ev_description="reschedule", ev_handler=schedule_next_pass,handler_arg = None,attr={})
	return True 
	
class BAN_LIST():
	def __init__(self,save_file=None):
		self.save_file = save_file
		self.list = []
		self.load()
		self.clean_old_passes()

	def save(self):
		if self.save_file is None : return
		try:
			with open(self.save_file, "wb") as f:
				pickle.dump(self.list, f)
		except Exception as e:
			print("Failed to save banned list: {}".format(e) )

	def load(self): #load persistent list or return a new empty one
		if self.save_file is None : return []
		try:
			with open(self.save_file, "rb") as f:
				self.list = pickle.load(f)
		except Exception as e:
			return []
	
	def clean_old_passes(self):
		ts = int(utime( datetime.utcnow()))
		i = 0
		while i < len( self.list):
			if self.list[i][0] < ts : del self.list[i]
			i += 1
		self.save()

	def ban_pass(self,passid):
		try:
			if self.is_banned(passid) : return #do not add duplicates
			#
			m=re.match('(\d\d\d\d\d\d\d\d_\d\d\d\d\d\d)_(.+)',passid) 
			if m is not None: 
				timestring = m.group(1)
				satname = m.group(2)
				ts = int(utime( datetime.strptime(timestring,"%Y%m%d_%H%M%S")))
				self.list.append([ts,satname,passid])
			else: print "ERROR in ban_pass(): passid does not match RE: {}".format(passid)
		except Exception as e:
			print "EXCEPTION in ban_pass(): {} : {}".format(passid,e)
		self.save()
	
	def unban_pass(self,passid): #unban pass if it is in the list ( effectively clean its value)
		try:
			m=re.match('(\d\d\d\d\d\d\d\d_\d\d\d\d\d\d)_(.+)',passid) 
			if m is not None: 
				timestring = m.group(1)
				satname = m.group(2)
				ts = int(utime( datetime.strptime(timestring,"%Y%m%d_%H%M%S")))
				#-- do search 
				for i in range(len( self.list)):
					if self.list[i][1] == satname:
						if abs(self.list[i][0] - ts) < 15*60 :  #pass is in the list
							del self.list[i]
							self.save()
							return True
			else: print "ERROR in unban_pass(): passid does not match RE: {}".format(passid)
		except Exception as e:
			print "EXCEPTION in unban_pass(): {} : {}".format(passid,e)
		self.save()
		return False
	
	def is_banned(self,passid): #check if pass is in the list
		try:
			m=re.match('(\d\d\d\d\d\d\d\d_\d\d\d\d\d\d)_(.+)',passid) 
			if m is not None: 
				timestring = m.group(1)
				satname = m.group(2)
				ts = int(utime( datetime.strptime(timestring,"%Y%m%d_%H%M%S")))
				#-- do search 
				for i in range(len( self.list)):
					if self.list[i][1] == satname:
						if abs(self.list[i][0] - ts) < 15*60 :  #pass is in the list
							return True
			else: print "ERROR in is_banned(): passid does not match RE: {}".format(passid)
		except Exception as e:
			print "EXCEPTION in is_banned(): {} : {}".format(passid,e)
		return False
	
	def dump(self):
		print( "-- ban list:")
		for p in self.list:
			print(" {} {} {}".format(p[0],p[1],p[2]))
			
class RTScheduler():
	"""A class which performs tasks at givel time moments"""
	tick_interval = RT_SCHEDULER_TICK
	SCHED_TABLE = {}
	def __init__(self):
		self._should_continue = False
		self.is_running = False
		self.thread = None
		self.default_handler = None

	def _handle_target(self):
		self.is_running = True
		self.schedule_tasks()
		self.is_running = False
		self._start_timer()

	def _start_timer(self):
		if self._should_continue: # Code could have been running when cancel was called.
			self.thread = threading.Timer(self.tick_interval, self._handle_target)
			self.thread.start()

	def start(self):
		if not self._should_continue and not self.is_running:
			self._should_continue = True
			self._start_timer()
		else:
			print("ERROR: Timer already started or running, please wait if you're restarting.")

	def cancel(self):
		if self.thread is not None:
			self._should_continue = False # Just in case thread is running and cancel fails.
			self.thread.cancel()
		else:
			print("ERROR:Timer never started or failed to initialize.")

	def add_event(self,ts,ev_description='generic event',ev_handler=None, handler_arg = None, attr = {}):
		ts_str = "{:%Y-%m-%d %H:%M:%S}".format(ts)
		self.SCHED_TABLE[ts_str] = { 'ts': ts, 'description' : ev_description, 'handler': ev_handler, 'handler_arg':handler_arg, 'attr':attr }
		logmsg("schedule event {} at {}".format(ev_description,ts))
	
	def remove_event(self,ts,ev_description=None):
		self.SCHED_TABLE.pop(ts_str)
		
	def default_reschedule(self):
		""" run default handler in case of errors """
		if self.default_handler is not None:
			time.sleep(2)
			logmsg("Running default handler")
			try:
				self.default_handler(None)
			except Exception as e:
				logmsg("EXCEPTION in default handler of : {}".format(e))
	
	def clean(self) : #clean all events
		ev_times = sorted(self.SCHED_TABLE)
		for ev_ts in ev_times:
			ev = self.SCHED_TABLE.pop(ev_ts)
			
	def schedule_tasks(self):
		#print "tick at {}".format(datetime.utcnow())
		#-- get first event
		ev_times = sorted(self.SCHED_TABLE)
		for ev_ts in ev_times:
			wait_time = self.SCHED_TABLE[ev_ts]['ts'] - datetime.utcnow()
			if wait_time.total_seconds() < self.tick_interval :
				#--run event handler
				ev = self.SCHED_TABLE.pop(ev_ts)  #this also removed event from the list !!
				logmsg("Running event [{}],ev_ts={}".format(ev['description'],ev_ts),level='INFO')
				if ev['handler'] is not None:
					try:
						res = ev['handler'](ev['handler_arg'])
						if res is None or not res:
							logmsg("Event hadler [{}] returned ERROR status".format(ev['description']))
							self.default_reschedule()
					except Exception as e:
						logmsg("EXCEPTION in handler of [{}] : {}".format(ev['description'],str(e)))
						self.default_reschedule()
		#
	
	def get_next_event(self): 
		ev_times = sorted(self.SCHED_TABLE)
		for ev_ts in ev_times:
			return self.SCHED_TABLE[ev_ts]
		return None

#-- setup logger --
#try:
#	loghandler = TimedRotatingFileHandler(os.path.join(APP_ROOT,'var', 'log',  "lora_"+CONF['station_id']+".log"), 
#		when='D', interval=1, backupCount=7, utc=True)
#	loghandler.setLevel(logging.DEBUG)
#	loghandler.setFormatter(logging.Formatter('%(asctime)s\t%(levelname)s\t%(message)s'))
#except Exception as e:
#	sys.stderr.write("WARNING: failed open file logger in [%s/log]:\n%s\n" % (APP_ROOT, str(e)))
#loghandler = logging.NullHandler

#-- configure app --
app = Flask(__name__)
app.config.conf = CONF
app.config.user = 'U'
BANNED_PASSES = BAN_LIST(save_file = os.path.join(APP_ROOT,'var','banned_passes.list'))
RTS = RTScheduler()
app.config.RTS = RTS
flask_log = logging.getLogger('werkzeug')
flask_log.setLevel(logging.ERROR)
#app.logger.addHandler(loghandler)
app.config['DEBUG'] = CONF['debug']
app.config['SECRET_KEY'] = 'fds5s5sejfd98034rt9u23h'

#dbg_toolbar = DebugToolbarExtension(app)

#--- login system --
login_manager = flask_login.LoginManager()
login_manager.init_app(app)
login_manager.login_view = 'login'

#--- some app globals 
STATIC_URL_ROOT = '/static' 

##@login_manager.user_loader 
def user_loader(uname):
	global CONF
	global STATIC_URL_ROOT
	STATIC_URL_ROOT = url_for('static', filename='')
	if uname not in CONF['USERS']:	
		print "-- Failed user_loader for " + str(uname)
		return None
	user = User(uname)
	user.is_authenticated = True
	g.current_user = uname
	return  user

##@login_manager.request_loader
def request_loader(request):
	uname = request.form.get('uname')
	global CONF
	if uname not in CONF['USERS']:   
		print "-- Failed request_loader for " + str(uname)
		return None
	user = User(uname)
	user.is_authenticated = True #request.form['pw'] == users[email]['pw']
	g.current_user = uname
	return user
	
#---------------------------------
#-- working classes ---
#------------------------------
class User():
	id = 'id'
	username = 'un'
	password = 'pw'
	email = 'em'
	is_authenticated = False
	
	def __init__(self , username):
		self.id = username
		self.username = username
		self.password = 'c'
		self.email =' m'
		self.registered_on = datetime.utcnow()
	
	def is_authenticated(self):
		return True

	def is_active(self):
		return True

	def is_anonymous(self):
		return False

	def get_id(self):
		return unicode(self.id)
	
	def __repr__(self):
		return '<User %r>' % (self.username)
		
#----- ---- --------- -------
#--  functions  --
#------ ----- ---- ----------
		
def get_exe_info_win(filename, info='FileVersion'):
    """ Extract information from executable file    """
    size = ctypes.windll.version.GetFileVersionInfoSizeA(filename, None)
    if not size:  return '-'
    res = ctypes.create_string_buffer(size)
    ctypes.windll.version.GetFileVersionInfoA(filename, None, size, res)
    r = ctypes.c_uint()
    l = ctypes.c_uint()
    ctypes.windll.version.VerQueryValueA(res, '\\VarFileInfo\\Translation', ctypes.byref(r), ctypes.byref(l))
    if not l.value:   return '--='
    # Take the first codepage (what else ?)
    codepages = array.array('H', ctypes.string_at(r.value, l.value))
    codepage = tuple(codepages[:2].tolist())
    # Extract information
    ctypes.windll.version.VerQueryValueA(res, ('\\StringFileInfo\\%04x%04x\\'+ info) % codepage,  ctypes.byref(r), ctypes.byref(l))
    return ctypes.string_at(r.value, l.value)
	
def get_system_info():
	SI = {}

	
	#
	SI['platform'] = platform.platform()
	SI['proc'] = platform.processor()
	SI['uname'] = ', '.join(s for s in platform.uname() )
	SI['os'] = platform.system()
	SI['os_version'] = """%s, v. %s,rel. %s""" % ( platform.system(),platform.release(),platform.version())
	SI['python_version'] = platform.python_version()
	SI['user'] =  os.getenv('USERNAME') or os.getenv('USER')
	SI['ip_list'] = '0.0.0.0'
	try:
		SI['ip_list'] = str([l for l in ([ip for ip in socket.gethostbyname_ex(socket.gethostname())[2] if not ip.startswith("127.")][:1], [[(s.connect(('8.8.8.8', 53)), s.getsockname()[0], s.close()) for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]][0][1]]) if l][0][0])
	except Exception as e:
		pass
	SI['hostname'] = platform.node()
	SI['fqdn'] = socket.getfqdn()
	tz = datetime.now() - datetime.utcnow()
	SI['TZ'] = int( 0.5 + tz.total_seconds() / 3600)
	#--- system specific values, initialized for notknown os case
	SI['storage'] = ''
	SI['processes'] = 'n/a'
	if SI['os'] == 'Windows':
		dletters = ['%s:' % d for d in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' if os.path.exists('%s:' % d)]
		for D in dletters:
			space = get_disk_space_kb(D )
			#space['total'] /= 1024*1024;
			#if (space['total'] < 1): space['total']=1 #to avoid division
			#space['free'] /= 1024*1024;
			#if space['free'] < 30 : space_style='err'
			#else : space_style='ok'
			SI['storage'] += " {} : {:.2f} GB <br />".format(D,space/10.0e6) ##(D, space['total'],space_style,space['free']*100/space['total'], space['free'])
		#--process info
		import subprocess
		wmic_cmd = 'WMIC PROCESS get Caption,Commandline,Processid'
		proc = subprocess.Popen(wmic_cmd, shell=True, stdout=subprocess.PIPE)
		SI['processes'] = ''
		
		#
		#SI['receiver_version'] = get_exe_info_win(os.path.join(CONF['ucp_dir'],'Receiver','Urmain.exe'))
		#SI['antenna_version'] = get_exe_info_win(os.path.join(CONF['ucp_dir'],'Antenna','Uamain.exe'))
		#SI['planner_version'] = get_exe_info_win(os.path.join(CONF['ucp_dir'],'Planner','Planner.exe'))

	return SI
	
def save_config():
	### save current system configuration 
	global conf_file
	try:
		json.dump(CONF, file(conf_file, 'w'))
	except Exception as e:
		
		return False
	return True

def get_client_ip():
	#-- TODO: - need to take into account reverse proxy , take proxy's headers
	ip = request.remote_addr
	return ip
	
def logmsg(msg, level='INFO'):
	username = '-'
	#if flask_login.current_user.is_authenticated: username = str(flask_login.current_user.get_id())
	#msg_txt = "%s@%s\t%s" % ('-', get_client_ip(), msg)
	msg_txt =  msg
	#-- simple log --
	logfile = LOG_FILE
	str = "{:%Y-%m-%d %H:%M:%S} | {}\t{}".format(datetime.utcnow(),level,msg)
	print str
	try:
		with open(logfile,'a') as lf:
			lf.write(str+"\n")
	except Exception as e:
		print "!! log write failed: {}\n!!{:%Y-%m-%d %H:%M:%S}\t{}\t{}\n".format(e,datetime.utcnow(),level,msg)

	
def parse_ini_lines(lines):
	ini = collections.OrderedDict()
	section_name='Main section'
	ini[section_name] = collections.OrderedDict()
	for ln in lines:
		m=re.match('^[#|;]',ln)
		if m is not None: continue #skip commented lines
		m=re.match('^\s*\[(\S.*)\]',ln) #new section
		if m is not None: 
			section_name = m.group(1)
			ini[section_name] = collections.OrderedDict()
			continue
		m=re.match('^\s*(\S.*)\s*=\s*(\S.*)',ln)
		if m is not None: ini[section_name][m.group(1)]=m.group(2)
	return ini
		
def read_ini(ini_name):
	#--get ini name
	ini_fn = ''
	lines = []
	ini = {}
	try:
		if ini_name == 'Planner': ini_fn = os.path.join(CONF['ucp_dir'],'Planner','Planner.ini')
		elif ini_name == 'Receiver': ini_fn = os.path.join(CONF['ucp_dir'],'Receiver','Urmain.ini')
		elif ini_name == 'Antenna': ini_fn = os.path.join(CONF['ucp_dir'],'Antenna','Uamain.ini')
		elif ini_name == 'Receiver_status': ini_fn = os.path.join(CONF['ucp_dir'],'Receiver','AppStatus.txt')
		elif ini_name == 'Antenna_status': ini_fn = os.path.join(CONF['ucp_dir'],'Antenna','AppStatus.txt')
		else : return {'status': 'error', 'ini': {}, 'msg':'Unknown INI ID', 'file_name' : '', 'lines':[]}
		print ">access ini : " + ini_fn
		with open(ini_fn) as fh:
			lines = fh.readlines()
			for ln in lines: ln.replace('\n|\r', '')
		ini = parse_ini_lines(lines)
	except Exception as e:
		logmsg("Failed to parse INI["+ini_name+"]:"+ini_fn+":"+ str(e),level='ERROR')
		return {'status': 'error', 'ini': {}, 'msg': "Exception: " + str(e), 'file_name' : ini_fn, 'lines':lines}
	return {'status': 'ok', 'ini': ini, 'msg':'readed INI ', 'file_name' : ini_fn, 'lines':lines}

def read_log(log_name):
	#--get log file name
	log_fn = ''
	lines = []
	try:
		if log_name == 'Planner_session': log_fn = os.path.join(CONF['ucp_dir'],'Planner','session.log')
		elif log_name == 'Planner_event': log_fn = os.path.join(CONF['ucp_dir'],'Planner','Event.log')
		elif log_name == 'urmain': log_fn = os.path.join(CONF['ucp_dir'],'Receiver','Urmain_Event.log')
		elif log_name == 'uamain': log_fn = os.path.join(CONF['ucp_dir'],'Antenna','Uamain.log')
		else : return {'status': 'error', 'log': {}, 'msg':'Unknown Log ID', 'file_name' : '', 'lines':[]}
		print ">access log : " +  log_fn
		with open(log_fn) as fh:
			lines = fh.readlines()
			for ln in lines: ln.replace('\n|\r', '')
		#ini = parse_ini_lines(lines)
		log = {}
	except Exception as e:
		logmsg("Failed to parse Log["+log_name+"]:"+log_fn+":"+ str(e),level='ERROR')
		return {'status': 'error', 'log': {}, 'msg': "Exception: " + str(e), 'file_name' : ini_fn, 'lines':lines}
	return {'status': 'ok', 'log': log, 'msg':'readed Log ', 'file_name' : log_fn, 'lines':lines}	

def get_sat_list() : 
	"""Return list of satellite names"""
	return SAT_PARAMS.keys()

def get_sat_options() : 
	"""Return list of satellite name options for use in HTML select"""
	opt = ''
	for s in get_sat_list():
		opt += "<option value='{s!s}'>{s!s}</option>".format(s=s)
	return opt
	
def encode_sat(name=''):
	""" Encode satellite name to 3-char code,  if input is 3 char long - it is already a code"""
	if len(name) == 3 : return name
	if name not in SAT_PARAMS: return 'XXX'
	return SAT_PARAMS[name]['id']

def calc_passesWW(satellite, tstart=None, tend=None):
	"""Calculate passes for given satellite over this station""" 
	#- 
	SCH = {'satellite': satellite,  'passes':[]}
	#-- read coordinates
	planner_ini =  read_ini('Planner')
	lat = float(planner_ini['ini']['Station']['Latitude\\Degrees'] )+float(planner_ini['ini']['Station']['Latitude\\Minutes'] )/60.0 
	if  planner_ini['ini']['Station']['Latitude\IsSouthHemisphere'] == 'true' : lat *= -1.0
	lon = float(planner_ini['ini']['Station']['Longitude\\Degrees'] )+float(planner_ini['ini']['Station']['Longitude\\Minutes'] )/60.0 
	if  planner_ini['ini']['Station']['Longitude\IsEastHemisphere'] == 'true' : lon *= -1.0
	SCH['lat'] = lat
	SCH['lon'] = lon
	#-- init orbit
	orb = Orbital(satellite)
	SCH['orb'] = str(orb)
	passes = orb.get_next_passes(datetime.utcnow(),SCHEDULE_EXTENT*24,lon,lat,1.0)
	SCH['orb'] 
	for apass in passes:
		SCH['passes'].append([ apass[0].strftime("%Y-%m-%d %H:%M:%S"),  apass[1].strftime("%Y-%m-%d %H:%M:%S"),  0 ])
	return SCH
	
	
def get_status():
	S = {}
	
	S['ts'] = "{:%H:%M:%S}".format(datetime.utcnow())
	#
	return S
	
def get_sessions():
	return None
	#--get decoded sessions ('past schedule') from the sessions log
	log_fn =  os.path.join(CONF['ucp_dir'],'Planner','session.log')
	lines = []
	sessions = []
	S={ 'date':'-', 'satellite':'-'} #session
	prev_pass_id = '-'
	try:
		with open(log_fn) as session_log:
			for ln in session_log:
				m=re.match('Current time.*(\d\d\d\d-\d\d-\d\d)',ln)
				if m is not None: S['date'] = m.group(1)
				m=re.match('Satellite:\s*(\S.*)',ln)
				if m is not None: S['satellite'] = m.group(1)
				m=re.match('Rev.*?(\d+)',ln)
				if m is not None: 
					S['revno'] = m.group(1)
					S['passid'] = S['satellite'] # + '_'+ S['revno']
				m=re.match('Schedule time.*(\d\d:\d\d:\d\d).*(\d\d:\d\d:\d\d)',ln)
				if m is not None: 
					S['start'] =  m.group(1)
					S['end'] =  m.group(2)
				m=re.match('^$|===+',ln)
				if m is not None: #this is a break after one pass block
					if S['satellite'] <> '-' and S['date'] <> '-' and S.has_key('passid'):
						if prev_pass_id <> S['passid']: #otherwise skip same pass records
							sessions.append(S)
							prev_pass_id = S.get('passid')
							S={ 'date':'-', 'satellite':'-'}
							
		
	except Exception as e:
		logmsg("Failed to parse sessions Log["+log_fn+"]:"+ str(e),level='ERROR')
		return None
	return sessions	
	
def get_scheduleXX(sched_id = ''):
	S = { 'has_schedule' : False , 'status' : 'undef', 'html' : '' }
	#--find the file
	if sched_id is None or sched_id == '': #select a defalt schedule
		sched_id = 'auto'
	fn = os.path.join(CONF['ucp_dir'],'Planner','Schedule_{0}.txt'.format(sched_id))
	S['file_name'] = fn
	S['has_schedule'] = True
	S['is_empty'] = True
	#-
	
	try:
		with open(fn) as fh:
			lines = fh.readlines()
	except Exception as e:
		logmsg("Read schedule exception in ["+fn+"]:"+ str(e),level='ERROR')
		S['status'] = 'error'
		S['has_schedule'] = False
		S['msg'] = "Read schedule exception in ["+fn+"]:"+ str(e)
		return S
	#
	S['html'] = """<table class='schedule'>\n""";
	try:
		prev_date = '0000-00-00'
		re_subline = re.compile(r'^ \S')
		re_parse = re.compile(r'(.+?)\s*\t\s*(\d+)\s+(\d+)\/(\d+)\/(\d+)\s+(\d\d:\d\d:\d\d)\s+(\d+)\s+(\d\d:\d\d:\d\d)\s+(\d+)\s+(\d+)\s+(\d\d:\d\d:\d\d)')
		now_time = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")
		dates_ref = ''
		for ln in lines:
			mp = re_parse.search(ln)
			pass_comment = ''
			if mp:
				s_sat = mp.group(1)
				s_orbit = int(mp.group(2))
				s_date = "%d-%s-%s" %(2000+int(mp.group(5)),mp.group(4),mp.group(3))
				s_t0 = mp.group(6)
				s_t1 = mp.group(11)
			else:
				#logmsg("Error parsing schedule line:"+ln,level='ERROR')
				continue
			#
			if prev_date <> s_date: #new date
				S['html'] += """<tr><td></td><td></td><td colspan=2><a name="D%s"></a><b>%s</b> <a href='#top'>up</a> | <a href='#bot'>down</a></td><td></td></tr>\n""" %(s_date,s_date)
				prev_date = s_date
				dates_ref += """|<a href='#D%s'>%s</a>""" % (s_date,s_date)
			#
			if (s_date+' '+s_t0) < now_time : #pass in the past
				tr_class = 'past'
				pass_comment = """<a href='plog/{sat}/{orbit}'> log</a>""".format(sat=s_sat, orbit=s_orbit)
			else: tr_class = 'future'
			#
			if (re_subline.match(ln)): 
				S['html'] += """<tr style='display:none'><td>===</td></tr>\n"""
			else: #this is a prime lines
				S['html'] += """<tr class='{trclass}'><td>{sat}</td><td class="ra">{orbit!s}</td><td>{t0}</td><td>{t1}</td><td>{comment}</td></tr>\n""".format(trclass=tr_class,
					sat=s_sat,orbit=s_orbit,t0=s_t0,t1=s_t1, 
					comment=pass_comment)
				#S['html'] += """<tr><td>%s</td><td>%d</td><td>%s</td><td></td></tr>\n""" % (s_sat,s_orbit,s_t0 )
		S['lines'] = lines
		S['status'] = 'ok'
		S['is_empty'] = len(lines)<1
		sched_selector = '' 
		for schtype in ('auto', 'manual', 'now','remote', 'send'): sched_selector+= """<a href="%s?id=%s"> %s </a> | """ % (url_for('schedule'), schtype, schtype)
		header = """<a name='top'></a>""" + dates_ref + "<br>\nThis is <b>"+sched_id+"</b> schedule, select another file: "+sched_selector+"<br><br>\n"
		footer = """ </table> \n<a name='bot'>.</a>\n"""
		S['html'] = header + S['html'] + footer;
	except Exception as e:
		logmsg("Schedule parsing exception in ["+fn+"]:"+ str(e),level='ERROR')
		S['msg'] = "Schedule parsing exception in ["+fn+"]:"+ str(e)
		S['status'] = 'error'
	return S

def get_future_schedule(sched_id = '', add_sat='', to_do = ''):
	S = { 'has_schedule' : False , 'status' : 'undef', 'html' : '' }
	#--find the file
	if sched_id is None or sched_id == '': #select a defalt schedule
		sched_id = 'now'
	fn = os.path.join(CONF['ucp_dir'],'Planner','Schedule_{0}.txt'.format(sched_id))
	S['file_name'] = fn
	S['has_schedule'] = True
	S['is_empty'] = True
	#-
	new_passes = None
	if to_do == 'addpass': #then calculate passes for given satellite
		if add_sat <> '-':
			new_passes = calc_passes(satellite=add_sat)
	#-
	try:
		with open(fn) as fh:
			lines = fh.readlines()
	except Exception as e:
		logmsg("Read schedule exception in ["+fn+"]:"+ str(e),level='ERROR')
		S['status'] = 'error'
		S['has_schedule'] = False
		S['msg'] = "Read schedule exception in ["+fn+"]:"+ str(e)
		return S
	#
	S['html'] = """<table class='schedule'>\n""";
	is_first_pass = True
	try:
		prev_date = '0000-00-00'
		re_subline = re.compile(r'^ \S')
		re_parse = re.compile(r'^(\S.+?)\s*\t\s*(\d+)\s+(\d+)\/(\d+)\/(\d+)\s+(\d\d:\d\d:\d\d)\s+(\d+)\s+(\d\d:\d\d:\d\d)\s+(\d+)\s+(\d+)\s+(\d\d:\d\d:\d\d)')
		now_time = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")
		dates_ref = ''
		i_new_pass = 0
		#------------ first traverse of the read schedule
		SCH = []
		for ln in lines:
			mp = re_parse.search(ln)
			pass_comment = ''
			if mp:
				s_sat = mp.group(1)
				s_orbit = int(mp.group(2))
				s_date = "%d-%s-%s" %(2000+int(mp.group(5)),mp.group(4),mp.group(3))
				s_t0 = mp.group(6)
				s_t1 = mp.group(11)
				s_dt0 = s_date + " " + s_t0
				s_dt1 = s_date + " " + s_t1
			else:
				#logmsg("Error parsing schedule line:"+ln,level='ERROR')
				continue
			#
			if  s_dt0 < now_time : continue  #drop all past passes
			
			if new_passes is not None: 
				#-- drop the same pass from the new candidates, done only for the same satellite
				if s_sat == add_sat:
					for idx, apass in enumerate(new_passes['passes']):
						if s_dt0 < apass[1] and s_dt1 > apass[0]: #overlapping times -> the same pass
							new_passes['passes'][idx] = [None, None, None] #remove passes already in the active schedule
				#-- look if we have any new pass to insert before this one
				while ( i_new_pass < len(new_passes['passes']) )   :
					if (new_passes['passes'][i_new_pass][0] is None) :
						i_new_pass += 1 #skip Nones
					else:
						if (new_passes['passes'][i_new_pass][0] < s_dt0):
							SCH.append({'is_new': True, 'dt0': new_passes['passes'][i_new_pass][0],  'sat':add_sat,  'orbit': str(new_passes['passes'][i_new_pass][2]),
								'date': new_passes['passes'][i_new_pass][0][0:10], 't0' :  new_passes['passes'][i_new_pass][0][10:19],  't1' :  new_passes['passes'][i_new_pass][1][10:19]})
							i_new_pass += 1
						else: break #leave i_new_pass at this position
					
			#
			SCH.append({'is_new': False,'dt0': s_dt0,  'sat':s_sat,  'orbit':s_orbit,'date': s_date, 't0' : s_t0,  't1' : s_t1})
							
		#----- second traverse and formatting, now uses combined SCH array
		for P in SCH:
			pass_comment = ''
			#
			if P['dt0'] < now_time : continue  #drop all past passes
			#
			s_date = P['date']
			if prev_date <> s_date : #new date
				S['html'] += """<tr><td colspan=3></td><td colspan=2><a name="D%s"></a><b>%s</b></td><td></td></tr>\n""" %(s_date,s_date)
				prev_date = s_date
				dates_ref += """|<a href='#D%s'>%s</a>""" % (s_date,s_date)
			#
			if (is_first_pass):
				S['html'] += """<tr ><td>=x=</td><td>===</td><td>===</td><td colspan='2' style='alighn:center;'> now: <span id='clock' style='font-style:bold'> -:-: </span></td><td>===</td></tr>\n"""
				is_first_pass = False
			#
			##passid = "{sat}_{orbit}_{sdate}_{t0}_{t1}".format(sat=s_sat,orbit=P['orbit'], t0=P['t0'], sdate=s_date, t1=P['t1'])
			passid = "{sat}_{sdate}_{t0}_{t1}".format(sat=s_sat,orbit=P['orbit'], t0=P['t0'], sdate=s_date, t1=P['t1'])

			#
			if P['is_new']: #this is a new proposed pass
				tr_class = 'new_pass'
				S['html'] += """<tr class='{trclass}'><td><input type='checkbox' id='{passid}' onclick='set_pass(this);' ></input></td><td>{sat}</td><td class="ra">{orbit!s}</td><td>{t0}</td><td>{t1}</td><td>{comment}</td></tr>\n""".format(trclass=tr_class,
					sat=P['sat'],orbit=P['orbit'],t0=P['t0'],t1=P['t1'],  passid=passid, 
					comment=pass_comment)
			else: #this is pass from existing schedule
				tr_class = 'x'
				S['html'] += """<tr class='{trclass}'><td><input type='checkbox' id='{passid}' checked  onclick='set_pass(this);'></input></td><td>{sat}</td><td class="ra">{orbit!s}</td><td>{t0}</td><td>{t1}</td><td>{comment}</td></tr>\n""".format(trclass=tr_class,
					sat=P['sat'],orbit=P['orbit'],t0=P['t0'],t1=P['t1'],  passid=passid, 
					comment=pass_comment)
				
		#-------------------------
		S['lines'] = lines
		S['status'] = 'ok'
		S['is_empty'] = len(lines)<1
		#-- header of the page 
		sat_selector = "<select id='satselect'><option value='-'>-select satellite-</option>"+get_sat_options()+"</select><button >Filter</button> <button onclick='update_schedule(\"addpass\");'>Add passes</button>"
		header = """<a name='top'></a>""" + dates_ref + "<br> This is <b>"+sched_id+"</b> schedule, <br>Satellite : "+sat_selector+" <br>\n"
		footer = """ </table> \n<a name='bot'>.</a>\n"""
		dbg_message = ''
		#if new_passes is not None: dbg_message = "<i>" + str(new_passes['passes']) + "</i>"
		S['html'] = header + S['html'] + footer + dbg_message;
	except ZeroDivisionError as e:
		logmsg("Schedule parsing exception in ["+fn+"]:"+ str(e),level='ERROR')
		S['msg'] = "Schedule parsing exception in ["+fn+"]:"+ str(e)
		S['status'] = 'error'
	return S
	
def format_ini_html(ini):
	html = "<div>\n"
	for section in ini['ini']:
		html += """<div class='section' onclick="toggle_block(this,'section_inner');">+ [ %s ]<div  class='section_inner' style="display:none" onclick="block_event(event);"><table class='section_table'>""" % (section)
		for par in ini['ini'][section]:
			html += "<tr><td class='par'>%s</td><td class='val'>%s</td><tr>" % (par,ini['ini'][section][par])
		html += "</table></div></div>\n"
	html += "</div> <!-- end of ini div -->\n"
	return html
	
def wrap_html_headers(content,title='LoreTT'):
	### add full HTML page header for over given body content
	head = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
    <title>%s</title>
    <link rel="stylesheet" href="/static/lora.css" type="text/css" />
	<script  src='/static/lora.js' type='text/javascript'></script>
</head>
<body> """ % (title)
	return head + content + "\n\n</body></html>\n"
	
def listDataFolders():
	""" list defined data folders and provide report on space usage """
	report = []
	rec_ini = read_ini('Receiver')
	if rec_ini['status'] != 'ok':
		logmsg("ERROR: listDataFolders(), failed to read Receiver INI, can not find data folders")
		return {}
	#list per satellite folder links
	for satid in rec_ini['ini']['DataRecord']:
		#--filter out unrelevant records
		satdir = rec_ini['ini']['DataRecord'][satid]
		m=re.search(r'\\',satdir) #has at least one slash, either / or \
		if m is not None:
			if satid not in ('RootDir','CustomDir','LogDir'): #explicit exclude of dir like values
				#-- check the folder
				total_size = 0
				nfiles = 0
				for dirpath, dirnames, filenames in os.walk(satdir):
					for f in filenames:
						fqfn = os.path.join(dirpath, f)
						total_size += os.path.getsize(fqfn)
						nfiles += 1
				report.append({ 'satellite' : satid, 'path' : satdir, 'nfiles' : nfiles, 'fsize' : total_size / (1024*1024)})
	return sorted(report,key=lambda x: x['fsize'], reverse=True)

def listDataDir(satid):
	""" profvides a listing of data directory, called by satellite name"""
	rec_ini = read_ini('Receiver')
	if rec_ini['status'] != 'ok':
		logmsg("ERROR: listDataDir(), failed to read Receiver INI, can not find data folders")
		return { 'status': 'error', 'msg': "Failed to access folder configuration files", 'ls' : [] }
	if not satid in rec_ini['ini']['DataRecord']:
		logmsg("ERROR: listDataDir(), failed to read Receiver INI, can not find data folders")
		return { 'status': 'error',  'msg': "Invalid satellite name", 'satellite': satid, 'folder': '' ,'ls' : [] }
	data_dir =  rec_ini['ini']['DataRecord'][satid]
	ls = []
	for dirpath, dirnames, filenames in os.walk(data_dir):
		for f in filenames:
			fqfn = os.path.join(dirpath, f)
			fsize = os.path.getsize(fqfn)
			fdate = datetime.fromtimestamp(os.path.getctime(fqfn)).replace(microsecond=0)
			#-- get file info
			m = re.search(r'^([^_]+)_(\d+)', f)
			if m:
				sat = m.group(1)
				orbit = m.group(2)
			else:
				sat = 'XXX'
				orbit = '00000'
			ls.append({'filename': f, 'fsize' :fsize, 'fdate': fdate,  'satellite':sat,'orbit':orbit} )
	return { 'status': 'ok',  'msg': "Listing of data folder for "+satid, 'satellite': satid, 'folder': data_dir ,
			'ls' : sorted(ls,key=lambda x: x['filename'], reverse=False) }
	
def format_folder_html(ls):
	html = """Data folder: <i>%s</i><br><form >
		<table class='def'><tr><th></th><th>File name</th><th>Size</th><th>Timestamp</th><th>Notes</th></tr>""" % ls['folder']

	for rec in ls['ls']:
		#--format some values
		if rec['fsize'] > 1024*1024 : fsize = str(rec['fsize']/(1024*1024)) + ' MiB'
		elif rec['fsize']>1024: fsize = str(rec['fsize']/(1024)) + ' kiB'
		else: fsize = str(rec['fsize']) + ' B'
		if isinstance(rec['fdate'], datetime) : fdate = rec['fdate'].isoformat()
		else: fdate = str(type(rec['fdate']))
		#
		plog_url = """<a href='{url}' target="_blank"> log</a>""".format( url=url_for('plog', sat=rec['satellite'],orbit= rec['orbit']) ) 
		html += "<tr><td><img source='%s' onclick='%s'><input type='checkbox' id=''/></td><td class='par'>%s</td><td class='ra'>%s</td><td class='ra'><i>%s</i></td><td>%s</td><tr>\n" \
				% (STATIC_URL_ROOT+'img/smi_delete.png','x',rec['filename'],fsize,fdate, plog_url)
	html += "</table></form>\n\n"
	return html


	
	
def do_quit():
	### Callback for quite of the server timer
	print "do_quit()"
	exit(1)

#-------------------------
#-- view routes --
#-------------------------
@app.route('/')
#@flask_login.login_required
def index():
	logmsg("/index",level='GET')
	return redirect(url_for('schedule')) 
	

@app.route('/login', methods=['GET', 'POST'])
def login():
	if request.method == 'GET':
		return render_template('login.htm',  error='')
	#else if this is a POST call, do authentication
	uname = request.form['uname']
	user = User(uname)
	print "trying to login " + uname
	flask_login.login_user(user)
	user.is_authenticated = True
	return redirect(url_for('index'))
	
@app.route('/logout')
#@flask_login.login_required
def logout():
	logmsg("/logout")
	logout_user()
	return redirect(url_for('login')) 

@app.route('/ping')
def ping():
	logmsg("/ping",level='GET')
	return "ping OK. this is " + CONF['station_id'] 

@app.route('/schedule')
#@flask_login.login_required
def schedule():
	logmsg("/schedule",level='GET' )
	#start = request.args.get('id','')
	 
	satlist = CONF['satlist']
	S = format_schedule(satlist,datetime.utcnow() - timedelta(hours=3*24),None)
	if S is None:
		return render_template('schedule.htm', status='error', msg='ERROR: calculating the schedule', menu_item='schedule',  error='', S=S)
	return render_template('schedule.htm', status='ok', msg='', menu_item='schedule',  error='', S=S)
	
@app.route('/schedule_new')
#@flask_login.login_required
def schedule_new():
	sched_id = request.args.get('id','')
	to_do = request.args.get('do','')
	satellites = request.args.get('sat','')
	logmsg("/schedule_new:"+str(sched_id) )
	schedule=get_future_schedule(sched_id, add_sat=satellites, to_do = to_do)
	return render_template('schedule_new.htm', menu_item='schedule',  error='', S=schedule)

@app.route('/schedule_past')
#@flask_login.login_required
def schedule_past():
	logmsg("/schedule_past")
	sessions=get_sessions()
	html= """ <a name='top'></a><table class="schedule">\n"""
	prev_date = '-'
	for s in sessions:
		tr_class = 'past'
		if prev_date <> s['date']: 
				html += """<tr><td></td><td></td><td colspan=2><a name="D%s"></a><b>%s</b></td><td><a href='#top'>up</a> | <a href='#bot'>down</a></td></tr>\n""" %(s['date'],s['date'])
				prev_date = s['date']
				
		html+= """<tr class='{trclass}'><td>{sat}</td><td class="ra">{orbit!s}</td><td>{t0}</td><td>{t1}</td><td></td></tr>\n""".format(trclass=tr_class,
					sat=s['satellite'],orbit=s['revno'],t0=s['start'],t1=s['end'] )
	html=html + """\n</table> <a name='bot'>.</a>\n"""
	return render_template('schedulepast.htm', menu_item='schedule',  error='', 
		S={'status':'ok','has_schedule':True,'html' : html})	
	
@app.route('/show_ini')
#@flask_login.login_required
def show_ini():
	global VALID_INI_ID
	ini_id = request.args.get('id','no-id')
	frmt = request.args.get('frmt','text')
	if ini_id in VALID_INI_ID:
		logmsg("/show_ini, INI ID="+ini_id)
		ini = read_ini(ini_id)
		if ini['status'] == 'ok':
			if frmt == 'html':
				tag_filelink = """<img src='%s' onclick="window.open('%s');">""" % (url_for('static',filename='img/icon_save.png'),url_for('show_ini') + '?frmt=plain&id='+ini_id)
				return wrap_html_headers("<pre>INI file: %s\nLocal file: %s %s <hr></span></pre>" %(ini_id,ini['file_name'],tag_filelink)  + format_ini_html(ini)  )
			if frmt == 'plain':
				return Response("".join( ini['lines'] ) , mimetype='text/plain')
			else:
				return "<pre>INI file: %s\nLocal file: %s<hr>\n\n%s</span></pre>" % (ini_id,ini['file_name'],pprint.pformat(ini['ini']) + str(ini['lines']))				
		else:
			return "<pre>INI file: %s\n<hr>\n\n<span class='error'>ERROR reading ini file [%s] : %s</span></pre>" % (ini_id,ini['file_name'],ini['msg'])
	else:
		logmsg("/show_ini, invalid ini ID="+ini_id,level='ERROR')
		return "<pre>INI file: <span class='error'>INVALID INI ID</span></pre>"

@app.route('/ini_files')
#@flask_login.login_required
def ini_files():
	logmsg("/ini_files")
	return render_template('ini_files.htm', menu_item='status',  error='', ini_list=VALID_INI_ID)

@app.route('/logs/scheduler')
#@flask_login.login_required	
def log_scheduler():
	logmsg("/logs")
	return render_template('logs.htm', menu_item='logs',  error='', log_files='scheduler')
	
@app.route('/logs')
#@flask_login.login_required
def logs():
	logmsg("/logs")
	return render_template('logs.htm', menu_item='logs',  error='', log_files=VALID_LOG_ID)

@app.route('/show_log')
#@flask_login.login_required
def show_log():
	global VALID_LOG_ID
	log_id = request.args.get('id','no-id')
	frmt = request.args.get('frmt','text')
	if log_id in VALID_LOG_ID:
		logmsg("/show_log, LOG ID="+log_id)
		log = read_log(log_id)
		if log['status'] == 'ok':
			if frmt == 'html':
				tag_filelink = """<img src='%s' onclick="window.open('%s');">""" % (url_for('static',filename='img/icon_save.png'),url_for('show_log') + '?frmt=plain&id='+log_id)
				log_text = ""
				re_errors = re.compile(r"ERROR", re.IGNORECASE)
				for ln in log['lines']:
					if (re_errors.search(ln)): 
						ln = "<span class='err'>"+ln+"</span>"
						print "ERRR"
					re.sub(r'[^\x00-\x7F]+','+',ln)
					log_text += ln 
				return wrap_html_headers("<pre>Log file: {0}\nLocal file: {1} {2} <hr></span><hr>{3}</pre>".format(log_id,log['file_name'],tag_filelink,log_text) )
			if frmt == 'plain':
				return Response("".join( log['lines'] ) , mimetype='text/plain')
			else:
				return Response("".join( log['lines'] ) , mimetype='text/plain')			
		else:
			return "<pre>LOG file: %s\n<hr>\n\n<span class='error'>ERROR reading log file [%s] : %s</span></pre>" % (log_id,log['file_name'],log['msg'])
	else:
		logmsg("/show_log, invalid log ID="+ini_id,level='ERROR')
		return "<pre>LOG file: <span class='error'>INVALID LOG ID</span></pre>"

@app.route('/viewfile/<filename>')
def viewfile(filename):
	#-- recognize file type
	file_dir = ''
	sfn = str(filename)
	if re.search(r"rec.log$",sfn):
		file_dir = CONF['track_archive']
		glob_mask = re.sub(r'^(.*\d\d\d\d\d\d\d\d_\d\d)\d\d\d\d_',r'\1*_',sfn) 
		g = glob.glob(os.path.join(file_dir,glob_mask))
		if len(g)<1 : return "Invalid file name"
		fpath = g[0]
	elif re.search("_bias_pass.txt$",sfn):
		file_dir = CONF['track_archive']
		glob_mask = re.sub(r'^(.*\d\d\d\d\d\d\d\d_\d\d)\d\d\d\d_',r'\1*_',sfn) 
		g = glob.glob(os.path.join(file_dir,glob_mask))
		if len(g)<1 : return "Invalid file name"
		fpath = g[0]
	elif sfn == 'syslog':
		fpath = LOG_FILE
	else:
		logmsg("unrecognized file case: [{}]".format(sfn),"ERROR")
		return "Invalid file name"
	#--get log file name
	log_fn = ''
	text = ''
	try:
		print ">open file for view : " +  fpath
		with open(fpath) as fh:
			lines = fh.readlines()
			for ln in lines:
				text += ln
	except Exception as e:
		logmsg("Failed to read file ["+fpath+"]:"+ str(e),level='ERROR')
		return "ERROR: failed to read file"
	return Response("#{}\n".format(fpath) + text,mimetype='text/plain')

	
	
@app.route('/data_files')
#@flask_login.login_required
def data_files():
	logmsg("/data_files")
	data_dir_report = listDataFolders()
	return render_template('data_files.htm', menu_item='data',  error='', dir_records=data_dir_report)

def find_plog_files(sat='',orbit=''):
	""" returns a list of pass log files for a given selection criteria"""
	plog_files = []
	#-0 - prepare data
	iorbit = int(orbit)
	orbit_5ch = "%05d"%iorbit
	glob_mask = "%s_%05d_*.log" % ( encode_sat(sat), iorbit)
	#-1-- collect a list of possible log directories
	check_dirs = []
	planner_ini =  read_ini('Planner')
	if planner_ini is not None and planner_ini['status'] == 'ok':
		log_root = planner_ini['ini']['Station']['LogFolder']
		for d in glob.glob(os.path.join(log_root, '*')) :
			if os.path.isdir(d) and not d in ('Antenna') : 
				check_dirs.append(d)
	#-TODO- check_dirs.append(os.path.join(CONF['ucp_dir'],'Receiver','Logs')) 
	#-2- check each dir 
	for log_dir in check_dirs:
		globls = glob.glob(os.path.join(log_dir,glob_mask))
		for f in globls: plog_files.append(f)
	#
	return plog_files
	
@app.route('/plog/<sat>/<orbit>')
#@flask_login.login_required
def plog(sat, orbit):
	""" Get pass log for given selection criteria"""
	format =  request.args.get('frmt','plain') 
	logmsg("/plog(sat=%s,orbit=%s,format=%s)"%(sat, orbit, format))
	#
	plog_files = find_plog_files(sat, orbit)
	#
	if format == 'plain':
		txt_lines =  []
		err = ''
		for plog_file in plog_files:
			#return Response(  str( type(plog_file)) , mimetype='text/plain')
			txt_lines += ["\n"+"*"*80 +"\n","*"*80 +"\n"]
			txt_lines.append("LOG FILE: "+plog_file)
			txt_lines.append("\n"+ "*"*80 +"\n")
			try:
				with open(plog_file) as fh:
					txt_lines += fh.readlines() 
			except Exception as e:
				err ="ERROR: Failed to read file["+plog_file+"]:"+ str(e)
				logmsg(err, level='ERROR')
				txt_lines.append("ERROR: "+err)
		return Response( "".join(txt_lines) , mimetype='text/plain')
	return render_template('plog.htm', menu_item='Logs',  error=strt(plogs))
	
@app.route('/show_datadir/<id>')
#@flask_login.login_required
def show_datadir(id):
	logmsg("/show_datadir/" + id)
	dir_list = listDataDir(id)
	return wrap_html_headers(format_folder_html(dir_list),"Data Folder: %s" %(dir_list['satellite']))

@app.route('/sysinfo')
#@flask_login.login_required
def sysinfo():
	logmsg("/sysinfo/")
	SI = get_system_info()
	return render_template('sysinfo.htm', menu_item='config',  error='', SI=SI, CONF=CONF)

@app.route('/setup')
#@flask_login.login_required
def setup():
	logmsg("/setup/")
	SI = {}
	SI['version'] = VERSION
	SI['start_time'] = "{:%Y-%m-%d %H:%M:%S}".format(g_start_time)
	SI['python_version'] = platform.python_version()
	SI['run_path'] = os.path.abspath(os.path.dirname(__file__))
	return render_template('setup.htm', menu_item='config',  error='', SI=SI)

@app.route('/con/frame')
#@flask_login.login_required
def con_get_frame():
	return render_template('conframe.htm', menu_item='tools',  error='')
	
@app.route('/con/screen')
#@flask_login.login_required
def con_get_screen():
	try:
		from PIL import ImageGrab
		shotfn = os.path.join(APP_ROOT,'log','screen.png')
		ImageGrab.grab().save(shotfn, "PNG")
		return send_file(shotfn, mimetype='image/png')
	except Exception as e:
		return render_template('gen_error.htm', message="System error, console access methods are not available", details=str(e))

@app.route('/con/click')
#@flask_login.login_required
def con_do_click():
	x = int(request.args.get('x','0'))
	y = int(request.args.get('y','0'))
	#TODO -- pywinauto.win32functions.SetForegroundWindow(pywinauto.findwindows.find_window(best_match='UniScan Receiver'))
	return Response( "got click at %d/%d"%(x,y) , mimetype='text/plain')

@app.route('/quit')
#@flask_login.login_required
def quit(): 
	### Terminate server process
	logmsg("/quit - SERVER IS COMMANDED TO TERMINATE")
	#threading.Timer(3,do_quit).start()
	flask_quit_func = request.environ.get('werkzeug.server.shutdown')
	if flask_quit_func is None:
		raise RuntimeError('Not running with the Werkzeug Server')
	flask_quit_func()
	return render_template('gen_error.htm', message="RECS server is going to shutdown in a few seconds. No more requests will be served",
	 details="")

@app.route('/export_schedule')
def export_schedule():
	""" track execution, sate the track text"""
	logmsg("/export_schedule")
	S = []
	for sat in CONF['satlist']: #collect schedules sat by sat
		ssat = get_sat_schedule(sat)
		S += ssat
		S.sort(key=lambda x: x['start'], reverse=False)
	#
	track_fn = ''
	sched_almur = format_almur_schedule(S)
	#
	try:
		track_fn = os.path.join(CONF['demod_path'],'schedule.ini')
		with open(track_fn, "w") as trackfile:
			trackfile.write(sched_almur)
	except Exception as e:
		return render_template('gen_error.htm', message="ERROR writing schedule to {}".format(track_fn), details=str(e))
	#
	return render_template('gen.htm', menu_item='test', message="Exported schedule".format(track_fn), details = "<pre>"+sched_almur+"</pre>")	
	
@app.route('/export_track/<satname>/<t0text>')
def export_track(satname,t0text):
	""" track execution, sate the track text"""
	logmsg("/export_track, sat={}, t0={}".format(satname,t0text))
	try:
		t0 = datetime.strptime(t0text,"%Y-%m-%d %H:%M:%S")
	except Exception as e:
		return render_template('gen.htm', menu_item='test', message="ERROR: invalitd time format", details = "")
	#
	scan_track = False
	if request.args.get('scan') is not None : scan_track = True
	# 
	s,track_start,bias_txt = get_track_text(satname,t0= t0 - timedelta(minutes=10),scan_track=scan_track) #datetime.utcnow() + timedelta(hours=10)
	pass_id = "{:%Y%m%d_%H%M%S}_{}".format(track_start,satname)
	track_fn = ''
	if s is not None:
		if 'track_export' in CONF:
			try:
				track_fn = os.path.join(CONF['track_archive'],pass_id+'.txt') 
				with open(track_fn, "w") as trackfile: #to archive
					trackfile.write(s)
				with open(CONF['track_export'] , "w") as trackfile: #to antenna
					trackfile.write(s)
				if bias_txt is not None and  len(bias_txt) > 10:
					with open(os.path.join(CONF['track_archive'],pass_id+'_bias.txt'), "w") as biasfile:
						biasfile.write(bias_txt)
			except Exception as e:
				return render_template('gen_error.htm', message="ERROR writing track to {}".format(track_fn), details=str(e))
	else:
		return render_template('gen_error.htm', message="Failed to calculate the track",details='')
	scan_txt = ''
	if scan_track : scan_txt = 'Scanning '
	return render_template('gen.htm', menu_item='test', message="{} Track file saved as [{}]".format(scan_txt,track_fn), details = "<pre>"+s+"</pre>")	 

def get_tle_set(satname,tle_file= None):
	""" Return dictionary of all satellites in the givel TLE file with their epochs """
	
	tleset = ''
	try:
		if tle_file is None: tle_file = get_tle()
		with open(tle_file, "r") as f: 
			tlelines = f.readlines()
			for i in range(0,len(tlelines)):
				m=re.match('^\s*#',tlelines[i]) #commented line
				if m is not None: continue
				m=re.match('^([^12].+?)\s*$',tlelines[i]) #not a TLE file - take it as a sat name
				if m is not None:
					if satname == m.group(1):
						tleset = tlelines[i+1] +tlelines[i+2]
	except Exception as e:
		print("Exception in get_tle_set(): {}".format(e))
	return tleset
	
def get_tle_epochs(tle_file= None):
	""" Return dictionary of all satellites in the givel TLE file with their epochs """
	if tle_file is None: tle_file = get_tle()
	TLE = {}
	satname = ''
	with open(tle_file, "r") as f: 
		for ln in f.readlines():
			m=re.match('^\s*#',ln) #commented line
			if m is not None: continue
			m=re.match('^([^12].+?)\s*$',ln) #not a TLE file - take it as a sat name
			if m is not None: 
				satname = m.group(1)
			else:
				m=re.match('^1 ...... ......   (\d\d)(\d\d\d).(\d+)',ln) #1st line in set
				if m is not None: 
					epoch = datetime(2000+int(m.group(1)), 1, 1) + timedelta(days = int(m.group(2)) - 1) + timedelta(seconds=int(3600*24*float("0."+m.group(3))))
					if satname in TLE:
						if epoch > TLE[satname]:
							TLE[satname]=epoch  #update if newer
					else:
						TLE[satname]=epoch
	return TLE


@app.route('/tle')
def tle():
	""" TLE info page  """
	logmsg("/tle");
	page = ""
	page += "<a href='{}'>Update TLE</a> || <a href='{}'>Current TLE file</a> <hr>\n".format(url_for('tle_update'),url_for('tle_get_current'))
	try:
		epochs = get_tle_epochs()
		page += "<table class='schedule'><tr><th>Satellite</th><th>Epoch</th><th>Age</th></tr>\n"
		for sat in sorted( CONF['satlist']):
			if sat in epochs:
				age = datetime.utcnow() - epochs[sat]
				age_days = age.total_seconds()/(24* 3600)
				if age_days > 5 : rec_class = 'error'
				elif age_days > 3 : rec_class = 'warning'
				else: rec_class = ''
				page += "<tr><td class='{rclass}'>{}</td><td class='{rclass}'>{}</td><td class='{rclass}'>{:.2f} days</td></tr>".format(sat,epochs[sat],age_days,rclass=rec_class)
			else:
				page += "<tr><td class='error'>{}</td><td class='error'>-- missing --</td><td class='error'>-</td></tr>".format(sat)
		page += "</table>"
	except Exception as e:
		logmsg("Exception in reading TLE, "+str(e),level="ERROR")
	return render_template('gen.htm', menu_item='tle', message='TLE orbital data', details = page)

@app.route('/tle/new.tle')
def tle_get_current():
	""" Return plain text of current TLE file, what ever it is """
	logmsg("/tle/new.tle")
	try:
		tle = get_tle()
		with open(tle, "r") as f:
			tle_text = f.read()
		return Response(tle_text, mimetype="text/plain")
	except Exception as e:
		return Response("ERROR: faied to read current tle file\n"+str(e), mimetype="text/plain")

def update_tle(source_id='default'):
	""" Update TLE file """
	sources = TLE_SOURCES[source_id]
	tle_text = "# Fetched from {}\n# At {}\n".format(source_id,datetime.utcnow())
	try:
		for src_url in sources:
			response = urllib2.urlopen(src_url)
			text =  response.read()
			logmsg("retrived TLE from [{}]".format(src_url))
			tle_text += text
	except Exception as e:
		logmsg("FAILED get TLE : {}".format(e))
		return False
	if len(tle_text) < 480:
		logmsg("FAILED get TLE : too short TLE retrieved, leave old version :"+str(len(tle_text)))
		return False
	#else - save new sets
	tle_fn = '--'
	try:
		tle_fn = get_tle()
		f = open(tle_fn, 'w')
		f.write(tle_text)
		f.close()
		# --archive a copy
		try:
			arch_tle = os.path.splitext(tle_fn)[0]+ "_{:%Y%m%d_%H%M}.tle".format(datetime.utcnow())
			shutil_copy(tle_fn,arch_tle)
		except Exception as ee:
			logmsg("Exception creationg archive copy [{}]:{}".format(tle_fn,str(ee)),level='ERROR')
			pass
	except Exception as e:
		logmsg("Exception writing new TLE to [{}]:{}".format(tle_fn,str(e)),level='ERROR')
		return False
	return True

def update_tle_periodic(arg):
	""" Update TLE file task for the scheduler"""
	try:
		logmsg("Starting periodic TLE update {}".format(str(arg)))
		update_tle()
		#schedule next run
		next_run_at = datetime.utcnow()+timedelta(hours=TLE_UPDATE_PERIOD_H)
		RTS.add_event(next_run_at,ev_description="TLE update",ev_handler=update_tle_periodic)
		logmsg("Scheduled next TLE update at {}".format(next_run_at))
	except Exception as e:
		logmsg("Exception in update_tle_periodic() :{}".format(str(e)),level='ERROR')
		return False
	return True
	
	
@app.route('/tle/update')
def tle_update():
	""" Return plain text of current TLE file, what ever it is """
	logmsg("/tle/update")
	update_tle()
	return render_template('gen.htm', menu_item='tle', message="OK. TLE file updated. <a href='{}'>Check TLE</a>".format(url_for('tle')), details = '')

@app.route('/events/get_next')
def events_get_next():
	""" """
	try:
		ev = RTS.get_next_event()
		if ev is None:
			return jsonify({'status':'no events','ts':'-','state':'-','info':'-'})
		#can not return ev directly, it is not serializable and need reformat of some values
		event= {}
		event['status'] = 'ok'
		if 'ts' in ev : event['ts'] = "{}".format(ev['ts'])
		else: event['ts'] = '-'
		event['info'] = ev['description'].replace('.',"<br />")
		if (ev['attr'] is not None) and 'state' in ev['attr']: #this is pass event, or at lest something with a state
			event['state'] = ev['attr']['state']
		else:
			event['state'] = '-'
		return jsonify(event)
	except Exception as e:
		return jsonify({'status':'error','message':"ERROR: failed to get current event\n"+str(e)})
	
def load_log(logfile):
	log = {}
	try:
		fields = []
		prev_t = ''
		with open(logfile,'rt') as F:
			for lnn in F:
				ln = lnn[:-1]
				if ln.count("\t") > 2: #data records
					if ln[0] == '#' or 'Time' in ln : #header line
						fields = ln.split("\t")
						##print "F={}".format(fields)
					else:
						r = ln.split("\t")
						if len(r) == len(fields):
							rec = {}
							rec['ts'] = r[0] #default
							t=r[0][0:8]
							if t <> prev_t:
								rec['ts'] = t
								prev_t = t
								for i in range(len(r)):
									rec[fields[i]] = r[i]
								#--customization of some fields or values
								if  ' OrbitAz' in rec: rec['OrbitAz'] = rec[' OrbitAz'] 
								if 'TargetAz' in rec:
									Daz = float(rec['TargetAz']) - float(rec['OrbitAz'])
									rec['Daz'] = "{:.2}".format(Daz)
									Del = float(rec['TargetEl']) - float(rec['OrbitEl'])
									rec['Del'] = "{:.2}".format(Del)
								##print "rec={}".format(rec)
								log[t] = rec
						else:
							print "={}={} <> {}".format(ln,len(r),len(fields))
		#
	except Exception as e:
		print "EXCEPTION in load_log({}) : {}".format(logfile,str(e))
	return log					
	
	
def join_logs(fid):
	""" return joined log from several files """
	joinedlog = []
	try:
		#--load track log
		glob_mask = os.path.join(CONF['track_archive'],re.sub(r'^(.*\d\d\d\d\d\d\d\d_\d\d)\d\d\d\d_([^_.]+).*',r'\1*\2*bias*',fid) )
		g = glob.glob(glob_mask)
		if len(g)>0 :
			print "+load log {}".format(g[0])
			track = load_log( g[0])
		else: 
			print "- no log file {}".format(glob_mask)
			track = {}
		#--load receiver log
		glob_mask = os.path.join(CONF['track_archive'],re.sub(r'^(.*\d\d\d\d\d\d\d\d_\d\d)\d\d\d\d_([^_.]+).*',r'\1*\2*rec.log*',fid) )
		g = glob.glob(glob_mask)
		if len(g)>0 :
			print "+load log {}".format(g[0])
			receiver = load_log( g[0])
		else: 
			print "- no log file {}".format(glob_mask)
			receiver = {}
		#-- join logs
		for ts in sorted( track.keys() ):
			if ts in receiver:
				rec = track[ts].copy()
				rec['ts'] = ts
				rec.update(receiver[ts])
				#
				joinedlog.append(rec)
				
	except Exception as e:
		print "EXCEPTION in join_logs : {}".format(str(e))
		return {}
	return joinedlog
		
@app.route('/joinlog/<fid>')
def joinlog(fid):
	fields = ['ts','TargetAz','TargetEl','Daz','Del','SNR']
	print "joinlog({})".format(fid)
	joinedlog = join_logs(fid)
	#
	logtext = ''# "{}".format(joinedlog)
	for f in fields: logtext += "{}\t".format(f)
	logtext += '\n'
	for r in joinedlog:
		try:
			ln = ''
			for f in fields: ln += "{}\t".format(r[f])
			logtext += ln + "\n"
		except Exception as e: 
			pass
	return  Response(logtext,mimetype='text/plain')

@app.route('/test')
def test():
	return  Response('test',mimetype='text/plain')
	
@app.route('/receiver')
def receiver():
	logmsg("/receiver");
	S = {}
	S['port'] = CONF['receiver']['man_port']
	R = almurnet.ALMUR_DEM('localhost',CONF['receiver']['man_port'])
	state = R.get_state()
	if state is None or state['status'] != 'OK': #error state
		return render_template('gen.htm', menu_item='receiver', message="Receiver ERROR. "  , details = state['status'])
	else:
		S['state'] = state
		return render_template('receiver.htm', menu_item='receiver', message="", S = S)	

@app.route('/trackview/<satname>/<reftime>')
def trackview(satname,reftime):
	""" return trackview page """
	try:
		if not satname in CONF['satlist'] : return jsonify({'status':'error','message':"Invalid satellite"})
		t0 = datetime.strptime(reftime,"%Y-%m-%dT%H:%M:%S")
		#
		S = { 'satellite':satname, 'track_table': '/tracktable/{}/{:%Y-%m-%dT%H:%M:%S}'.format(satname,t0),
			'rssi_table': '/receiver/reclog/{:%Y%m%d_%H%M%S}_{}'.format(t0,satname),
			'min_elevation':CONF['min_elevation'], 'pass_start':"{}".format(t0)}
	except Exception as e:
		render_template('gen.htm', menu_item='test', message="test", details = "ERROR: {}".format(e))
	return render_template('trackview.htm', menu_item='receiver', message="", S = S)	
	
@app.route('/trackview/<passid>')
def trackview_bypass(passid):
	""" return trackview page """
	try:
		m = re.search(r'^(.*\d\d\d\d\d\d\d\d_\d\d\d\d\d\d)_([^_.]+)',passid)
		if m is None: return jsonify({'status':'error','message':"invalid passid"})
		satname = m.group(2)
		t0 = datetime.strptime(m.group(1),"%Y%m%d_%H%M%S")
		if not satname in CONF['satlist'] : return jsonify({'status':'error','message':"Invalid satellite"})
		#
		S = { 'satellite':satname, 'track_table': '/tracktable/-/{}'.format(passid),
			'rssi_table': '/receiver/reclog/{}'.format(passid),
			'min_elevation':CONF['min_elevation'], 'pass_start':"{}".format(t0)}
	except Exception as e:
		render_template('gen.htm', menu_item='test', message="test", details = "ERROR: {}".format(e))
	return render_template('trackview.htm', menu_item='receiver', message="", S = S)
	
@app.route('/receiver/reclog/<passid>')
def rec_reclog(passid):
	""" return tabled log of the receiver """
	rssi_table = []
	glob_mask = os.path.join(CONF['track_archive'],re.sub(r'^(.*\d\d\d\d\d\d\d\d_\d\d)\d\d\d\d_([^_.]+).*',r'\1*\2*rec.log',passid) )
	g = glob.glob(glob_mask)
	if len(g)<1 :
		print "+failed to glob for rec file  {}".format(glob_mask)
		return jsonify({'status':'error','message':"Failed to glob rec log", 'rssi_table':rssi_table})
	rssi_min = 0
	rssi_max = -100
	try:
		with open(g[0],'rt') as F:
			##print "reading reclog {}".format(g[0])
			for lnn in F:
				ln = lnn[:-1]
				if ln.count("\t") > 2: #data records
					if ln[0] == '#' or 'Time' in ln : #header line
						pass
					else:
						r = ln.split("\t")
						##print r
						try:
							ts =  datetime.strptime(r[0][0:19],"%Y-%m-%d %H:%M:%S")
							rssi = float(r[3]) if len(r)<5 else float(r[4])
							rssi_table.append({'t':utime(ts),'ts':"{:%H:%M:%S}".format(ts),'rssi':float("{}".format(rssi))}) #
							if rssi > rssi_max : rssi_max = rssi
							if rssi < rssi_min : rssi_min = rssi
						except Exception as ee:
							print "parsing exception in ln: {}:{}".format(ln,ee)
							
	except Exception as e:
		print "EXCEPTION in rec_reclog({}) : {}".format(passid,e)
		return jsonify({'status':'error','message':"Failed to load rec log", 'rssi_table':rssi_table})
	if rssi_min > -10: rssi_min = 0.0
	return jsonify({'status':'ok','message':"", 'rssi_table':rssi_table, 'rssi_min':rssi_min, 'rssi_max':rssi_max})
		
@app.route('/receiver/rssitable')
def rec_rssi_table():
	""" return current realtime rssi log """
	rssi_table = []
	if RADIO is not None: 
		rt = RADIO.rssi_log
		for r in rt:
			rssi_table.append({'t':utime(r[0]),'ts':"{:%H:%M:%S}".format(r[0]),'rssi':float("{:.2f}".format(r[1]))})
	return jsonify({'status':'ok','message':"", 'rssi_table':rssi_table})
	
@app.route('/receiver/rssi')
def rec_rssi():
	""" return current rssi value """
	rssi = 0.0
	ut = utimedatetime.utcnow()
	if RADIO is not None: 
		rssi = RADIO.rssi
	return jsonify({'status':'ok','message':"", "utime": ut, "ts" : "{}".format(datetime.utcnow()), 'rssi':rssi})	
	
@app.route('/tracktable/<satname>/<reftime>')
def tracktable(satname,reftime):
	""" Return track in timestamped JSON array. Table structure: TSfloat,TShours,AZ,EL """
	try:
		tracktable = []
		if satname =='-': # call by pass id
			m = re.search(r'^(.*\d\d\d\d\d\d\d\d_\d\d\d\d\d\d)_([^_.]+)',reftime)
			if m is None: return jsonify({'status':'error','message':"invalid passid"})
			satname = m.group(2)
			t0 = datetime.strptime(m.group(1),"%Y%m%d_%H%M%S")
		else:
			t0 = datetime.strptime(reftime,"%Y-%m-%dT%H:%M:%S")
		if not satname in CONF['satlist'] : return jsonify({'status':'error','message':"Invalid satellite"})
		#
		tle = get_tle(satname,t0)
		orb = Orbital(satname, tle_file=tle)
		#
		passes = orb.get_next_passes(t0-timedelta(minutes=20),1,CONF['station_lon'],CONF['station_lat'],CONF['station_alt'],tol=0.01, horizon=CALC_MIN_ELEV) 
		if len(passes)<1: return jsonify({'status':'error','message':"No passes at ref.time {}".format(t0)})
		next_pass =  passes[0]
		pass_start = next_pass[0].replace(microsecond=0)
		pass_end = next_pass[1].replace(microsecond=0)
		orbnum = orb.get_orbit_number(next_pass[2])
		#
		dt=0
		t = pass_start
		ut = utime(pass_start)
		while t < pass_end:
			az, el = orb.get_observer_look(t, CONF['station_lon'],CONF['station_lat'], 0)
			if el > CONF['min_elevation']:
				tracktable.append({'t':ut,'ts':"{:%H:%M:%S}".format(t),'az':float("{:.2f}".format(az)),'el':float("{:.2f}".format(el))})
			dt+=1
			ut+=1
			t = pass_start + timedelta(seconds=dt)
		#
		return jsonify({'status':'ok','message':"",'satellite':satname, 'orbit':orbnum, 
				'pass_start':"{:%Y-%m-%d %H:%M:%S}".format(pass_start), 'pass_end':"{:%Y-%m-%d %H:%M:%S}".format(pass_end),'table':tracktable})
	except Exception as e:
		return jsonify({'status':'error','message':"EXCEPTION: failed to get current event\n"+str(e)})
		
#-------------------------

@app.route('/banpass/<passid>')
def banpass(passid):
	logmsg("banning pass {}".format(passid))
	BANNED_PASSES.ban_pass(passid)
	schedule_next_pass(None, clean_all_events=True)
	return Response("OK",mimetype='text/plain')

@app.route('/unbanpass/<passid>')
def unbanpass(passid):
	logmsg("banning pass {}".format(passid))
	BANNED_PASSES.unban_pass(passid)
	schedule_next_pass(None, clean_all_events=True)
	return Response("OK",mimetype='text/plain')

@app.route('/banunbanpass/<banpassid>/<unbanpassid>')
def banunbanpass(banpassid,unbanpassid): #switch between conflicting passes
	logmsg("switch conflicting passes, ban: {} , use: {}".format(banpassid,unbanpassid))
	BANNED_PASSES.ban_pass(banpassid)
	BANNED_PASSES.unban_pass(unbanpassid)
	schedule_next_pass(None, clean_all_events=True)
	return Response("OK",mimetype='text/plain')

#--  main part of the executable  ---

def init_main(): 
	""" Some initi oprations, to be called fro mmain() """
	#-- prepare some folders
	need_dirs = [APP_ROOT+'/var/log',APP_ROOT+'/var/tle']
	try:
		for d in need_dirs:
			print "mkdir "+d
			if not os.path.exists(d): os.makedirs(d)
	except Exception as e:
		print "ERROR: failed to create system dir [{}] : {}".format(d,srtrt(e))
	
	
	
if __name__ == '__main__':
	logmsg("Starting...")
	#####
	if not app.debug or os.environ.get("WERKZEUG_RUN_MAIN") == "true":
		logmsg("Initializing application "+ __file__)
		init_main()
		#
		RTS.start()
		RTS.default_reschedule =  schedule_next_pass
		try:
			SCHED = calc_schedule(t0=datetime.utcnow() - timedelta(seconds=15*60))
		except Exception as e: pass
		RTS.add_event(datetime.utcnow()+timedelta(seconds=3),ev_description="Initial scheduling",ev_handler=schedule_next_pass)
		#RTS.add_event(datetime.utcnow()+timedelta(seconds=1000),ev_description="TLE update",ev_handler=update_tle_periodic)
		
		#
		#print(logging.Logger.manager.loggerDict)
	os.environ['PYTHONPATH'] = os.getcwd() #-- this lets the reloader to work properly
	app.run(host='0.0.0.0',port=80, use_reloader=True, threaded=True)

""" 
HISTORY

1.5.20200531
add  default_reschedule to the scheduler ( to call in case of event failures. 
Workaround for 23:59:59 issue

"""
